package roneditor.gef.commands.objectnet;

import java.util.List;
import java.util.Vector;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CompoundCommand;

import roneditor.gef.commands.rule.DeleteMappingCommand;
import roneditor.model.RuleUtil;
import ronmodel.Codomain;
import ronmodel.LHS;
import ronmodel.Mapping;
import ronmodel.ONArc;
import ronmodel.ONNode;
import ronmodel.ObjectNet;
import ronmodel.Rule;

/**
 * The <code>DeleteONNodeCompoundCommand</code> is used to delete an
 * {@link ONNode} and all the connected {@link ONArc}. Additionally, if the
 * {@link ONNode} is part of an {@link Rule}, all associated {@link Mapping}
 * will be deleted.
 */
public class DeleteNodeCommand extends CompoundCommand {
	public static final String LABEL = "DeleteNodeCommand";

	/**
	 * The inner class <code>DeleteONNodeCommand</code> is part of the
	 * compound and deletes the giben {@link ONNode} with no further checks.
	 */
	private class DeleteONNodeCommand extends Command {
		public static final String LABEL = "DeleteONNodeCommand";

		private ONNode node;

		private ObjectNet objectNet;

		public DeleteONNodeCommand() {
			super(LABEL);
		}

		public DeleteONNodeCommand(ONNode node) {
			super(LABEL);
			this.node = node;
			this.objectNet = node.getObjectNet();
		}

		@Override
		public boolean canExecute() {
			return (node != null);
		}

		@Override
		public void execute() {
			node.setObjectNet(null);
		}

		@Override
		public void redo() {
			node.setObjectNet(null);
		}

		@Override
		public void undo() {
			node.setObjectNet(objectNet);
		}
	}

	@SuppressWarnings("unchecked")
	public DeleteNodeCommand(ONNode node) {
		super(LABEL);

		// Delete Mappings
		ObjectNet objectNet = node.getObjectNet();
		if (objectNet instanceof LHS || objectNet instanceof Codomain) {
			List<Mapping> mappings;
			if (objectNet instanceof LHS) {
				mappings = RuleUtil.getAllMappings(((LHS) objectNet).getRule());
			} else {
				mappings = ((Codomain) objectNet).getMappings();
			}
			for (Mapping mapping : mappings) {
				if (mapping.getTarget() == node || mapping.getSource() == node) {
					add(new DeleteMappingCommand(mapping));
				}
			}
		}

		// Delete ONArcs
		Vector<ONArc> incomingArcs = new Vector<ONArc>(node.getIncoming());
		for (ONArc link : incomingArcs) {
			add(new DeleteArcCommand(link));
		}
		Vector<ONArc> outgoingArcs = new Vector<ONArc>(node.getOutgoing());
		for (ONArc link : outgoingArcs) {
			add(new DeleteArcCommand(link));
		}

		// Delete ONNode itself
		add(new DeleteONNodeCommand(node));
	}

}
