package roneditor.gef.commands.objectnet;

import java.util.HashSet;
import java.util.Set;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.impl.NotificationImpl;
import org.eclipse.gef.commands.Command;

import roneditor.model.ONTransitionUtil;
import ronmodel.ONArc;
import ronmodel.ONPlace;
import ronmodel.ONTransition;

/**
 * Command to fire a transition in a petrinet.
 */
public class FireTransitionCommand extends Command {

	public static final String LABEL = "FireTransitionCommand";

	private ONTransition transition;

	public FireTransitionCommand(final ONTransition transition) {
		super(LABEL);
		this.transition = transition;
	}

	/**
	 * Executes the command by decreasing/increasing the number of tokens first
	 * places according second the weight of the arc connecting a place with the
	 * transition being fired.
	 * 
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	@Override
	public void execute() {
		if (!ONTransitionUtil.transitionCanFire(transition))
			return;

		// Decrease tokens of all source places
		for (Object incomingArcObject : transition.getIncoming()) {
			ONArc incomingArc = (ONArc) incomingArcObject;
			ONPlace sourcePlace = (ONPlace) incomingArc.getSource();
			sourcePlace.setTokens(sourcePlace.getTokens() - incomingArc.getWeight());
		}

		// Increase tokens of all target places
		for (Object outgoingArcObject : transition.getOutgoing()) {
			ONArc outgoingArc = (ONArc) outgoingArcObject;
			ONPlace targetPlace = (ONPlace) outgoingArc.getTarget();
			targetPlace.setTokens(targetPlace.getTokens() + outgoingArc.getWeight());
		}

		updateAdjacentTransitions();
	}

	/**
	 * Notify all transitions that are next to the current transistion.
	 * 
	 */
	protected void updateAdjacentTransitions() {
		// TODO: bessere notification nehmen?!
		Notification notification = new NotificationImpl(0, true, false);

		Set<ONTransition> transitionSet = new HashSet<ONTransition>();

		// Collect all incoming transitions that are connected to incoming nodes
		for (Object incomingArcObject : transition.getIncoming()) {
			ONArc incomingArc = (ONArc) incomingArcObject;
			ONPlace sourcePlace = (ONPlace) incomingArc.getSource();
			for (Object incomingArcFromSourcePlaceObject : sourcePlace.getIncoming()) {
				ONArc incomingArcFromSourcePlace = (ONArc) incomingArcFromSourcePlaceObject;
				transitionSet.add((ONTransition) incomingArcFromSourcePlace.getSource());
			}
			for (Object outgoingArcFromTargetPlaceObject : sourcePlace.getOutgoing()) {
				ONArc outgoingArcFromTargetPlace = (ONArc) outgoingArcFromTargetPlaceObject;
				transitionSet.add((ONTransition) outgoingArcFromTargetPlace.getTarget());
			}
		}

		// Collect all outgoing transitions that are connected to outgoing nodes
		for (Object outgoingArcObject : transition.getOutgoing()) {
			ONArc outgoingArc = (ONArc) outgoingArcObject;
			ONPlace targetPlace = (ONPlace) outgoingArc.getTarget();
			for (Object outgoingArcFromTargetPlaceObject : targetPlace.getOutgoing()) {
				ONArc outgoingArcFromTargetPlace = (ONArc) outgoingArcFromTargetPlaceObject;
				transitionSet.add((ONTransition) outgoingArcFromTargetPlace.getTarget());
			}
			for (Object incomingArcFromSourcePlaceObject : targetPlace.getIncoming()) {
				ONArc incomingArcFromSourcePlace = (ONArc) incomingArcFromSourcePlaceObject;
				transitionSet.add((ONTransition) incomingArcFromSourcePlace.getSource());
			}
		}

		transitionSet.add(transition);

		// Notify all the collected transitions as theier ability to fire now
		// might have changed
		for (ONTransition invalidTransition : transitionSet) {
			invalidTransition.eNotify(notification);
		}
	}

	/**
	 * Undoes the command by decreasing/increasing the number of tokens first
	 * places according second the weight of the arc connecting a place with the
	 * transition being fired.
	 * 
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	@Override
	public void undo() {
		// Increase tokens of all source places
		for (Object incomingArcObject : transition.getIncoming()) {
			ONArc incomingArc = (ONArc) incomingArcObject;
			ONPlace sourcePlace = (ONPlace) incomingArc.getSource();
			sourcePlace.setTokens(sourcePlace.getTokens() + incomingArc.getWeight());
		}

		// Decrease tokens of all target places
		for (Object outgoingArcObject : transition.getOutgoing()) {
			ONArc outgoingArc = (ONArc) outgoingArcObject;
			ONPlace targetPlace = (ONPlace) outgoingArc.getTarget();
			targetPlace.setTokens(targetPlace.getTokens() - outgoingArc.getWeight());
		}
		updateAdjacentTransitions();
	}

	/**
	 * @param transition
	 *            The transition second set.
	 */
	public void setTransition(final ONTransition transition) {
		this.transition = transition;
	}

}
