/**
 * 
 */
package roneditor.gef.commands.objectnet;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.draw2d.graph.DirectedGraph;
import org.eclipse.draw2d.graph.DirectedGraphLayout;
import org.eclipse.draw2d.graph.Edge;
import org.eclipse.draw2d.graph.EdgeList;
import org.eclipse.draw2d.graph.Node;
import org.eclipse.draw2d.graph.NodeList;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.Command;

import roneditor.gef.editparts.objectnet.ONNodeEditPart;
import roneditor.gef.editparts.objectnet.ObjectNetEditPart;
import ronmodel.ONArc;
import ronmodel.ONNode;

/**
 * The {@link ObjectNetEditPartGraphLayoutCommand} class provides an {@link Command}
 * that aligns the graph according to the Draw2d method
 * {@link DirectedGraphLayout}. It will transform the given sizes and positions
 * in the {@link EditPart}s and it's model to the appropriate Draw2d Node/Edge structure and
 * apply the generated new positions to the model.
 */
public class ObjectNetEditPartGraphLayoutCommand extends Command {

	public static final String LABEL = "ObjectNetEditPartGraphLayoutCommand";

	private ObjectNetEditPart objectnetEditPart;

	private HashMap<ONNode, Point> oldPositions;

	private HashMap<ONNode, Point> newPositions;

	/** Default padding for nodes in the graph (fixed spacing around nodes). */
	protected final int DEFAULT_PADDING = 25;

	/**
	 * Constructor needs an {@link ObjectNetEditPart}.
	 * 
	 * @param objectnetEditPart
	 */
	public ObjectNetEditPartGraphLayoutCommand(ObjectNetEditPart objectnetEditPart) {
		super(LABEL);
		this.objectnetEditPart = objectnetEditPart;
		oldPositions = new HashMap<ONNode, Point>();
		newPositions = new HashMap<ONNode, Point>();
	}

	/**
	 * Apply the stored oldPositions to the Model to revert the changes
	 */
	@SuppressWarnings("unchecked")
	public void undo() {
		List<ONNode> onNodeList = objectnetEditPart.getCastedModel().getNodes();
		for (ONNode onNode : onNodeList) {
			Point p = oldPositions.get(onNode);
			if (p != null) {
				onNode.setX(p.x);
				onNode.setY(p.y);
			}
		}
	}

	/**
	 * Redo will simply apply the stored positions again
	 */
	public void redo() {
		applyNewPositions();
	}

	/**
	 * Apply the stored newPositions to the Model
	 */
	@SuppressWarnings("unchecked")
	protected void applyNewPositions() {
		List<ONNode> onNodeList = objectnetEditPart.getCastedModel().getNodes();
		for (ONNode onNode : onNodeList) {
			Point p = newPositions.get(onNode);
			if (p != null) {
				onNode.setX(p.x);
				onNode.setY(p.y);
			}
		}
	}

	/**
	 * From the stored {@link ObjectNetEditPart} the newPositions HashMap will
	 * be created according to the Positions generated from the
	 * {@link DirectedGraphLayout}.
	 */
	@SuppressWarnings("unchecked")
	public void buildGraphLayout() {
		DirectedGraph directedGraph = new DirectedGraph();
		directedGraph.edges = new EdgeList();
		directedGraph.nodes = new NodeList();

		// Get the Bounds for the Nodes from the view
		HashMap<ONNode, Rectangle> boundsMap = new HashMap<ONNode, Rectangle>();
		for (Object child : objectnetEditPart.getChildren()) {
			if (child instanceof ONNodeEditPart) {
				ONNodeEditPart onNodeEditPart = (ONNodeEditPart) child;
				ONNode onNode = (ONNode) (onNodeEditPart.getModel());
				boundsMap.put(onNode, onNodeEditPart.getFigure().getBounds());
			}
		}

		List<ONNode> onNodeList = objectnetEditPart.getCastedModel().getNodes();
		List<ONArc> onArcList = objectnetEditPart.getCastedModel().getArcs();
		Map<ONNode, Node> onNodeToNodeMap = new HashMap<ONNode, Node>();

		// Convert ONNodes (plus the Bounds from the View) to (Draw2d) Nodes
		for (ONNode onNode : onNodeList) {
			Node n = new Node(onNode);
			n.x = onNode.getX();
			n.y = onNode.getY();
			Rectangle bounds = boundsMap.get(onNode);
			if (bounds != null) {
				n.width = bounds.width;
				n.height = bounds.height;
			} else {
				// Actually this should not happen. Just in case we will just
				// assume the size is 100x100
				System.err.println("WARNING: The ONNode " + onNode + " has no Bounding Rectangle attached");
				n.width = 100;
				n.height = 100;
			}
			directedGraph.nodes.add(n);
			onNodeToNodeMap.put(onNode, n);
			oldPositions.put(onNode, new Point(onNode.getX(), onNode.getY()));
		}

		// Convert ONArcs to (Draw2d) Edges
		for (ONArc onArc : onArcList) {
			directedGraph.edges.add(new Edge(onNodeToNodeMap.get(onArc.getSource()), onNodeToNodeMap.get(onArc.getTarget())));
		}

		directedGraph.setDefaultPadding(new Insets(DEFAULT_PADDING));

		// Perform Layout
		new DirectedGraphLayout().visit(directedGraph);

		// Store new positions
		for (Object nodeObject : directedGraph.nodes) {
			Node node = (Node) nodeObject;
			newPositions.put((ONNode) (node.data ), new Point(node.x, node.y));
		}

	}

	public void execute() {
		buildGraphLayout();
		applyNewPositions();
	}

}
