package roneditor.gef.commands.ron;

import java.util.Collection;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CompoundCommand;

import roneditor.model.ObjectNetUtil;
import ronmodel.HLArc;
import ronmodel.HLPlace;
import ronmodel.HLPlaceType;
import ronmodel.HLTransition;
import ronmodel.ONNode;
import ronmodel.ObjectNet;
import ronmodel.Token;

/**
 * The {@link HLTransitionSplitCommand} is used when we fire an
 * {@link HLTransition} of type <code>SPLIT</code>. We can only execute it
 * when there is one incoming {@link HLPlace} of type <code>NET</code> and one
 * outgoing {@link HLPlace} of type <code>NET</code>. The
 * {@link CompoundCommand} will then split the {@link Token} (which are in turn
 * an {@link ObjectNet}) placed on the source {@link HLPlace} in it's connected
 * component and put the generated {@link ObjectNet}s (which are in turn
 * {@link Token}s again) on the target {@link HLPlace}.
 */
public class HLTransitionSplitCommand extends CompoundCommand {

	public static final String LABEL = "SplitObjectNetCommand";

	protected class SplitObjectNetCommand extends Command {

		HLPlace sourcePlace;

		HLPlace targetPlace;

		ObjectNet sourceToken;

		Collection<ObjectNet> targetToken;

		SplitObjectNetCommand(ObjectNet token, HLPlace target) {
			this.sourceToken = token;
			this.targetPlace = target;
			this.sourcePlace = token.getPlace();
		}

		public void execute() {
			targetToken = ObjectNetUtil.split(sourceToken);
			for (ObjectNet net : targetToken) {
				net.setPlace(targetPlace);
			}
			sourceToken.setPlace(null);
		}

		public void redo() {
			execute();
		}

		@SuppressWarnings("unchecked")
		public void undo() {
			for (ObjectNet net : targetToken) {
				Collection<ONNode> nodes = net.getNodes();
				ObjectNetUtil.moveConnectedComponent(nodes, sourceToken);
				net.setPlace(null);
			}
			sourceToken.setPlace(sourcePlace);
		}

	}

	@SuppressWarnings("unchecked")
	public HLTransitionSplitCommand(HLTransition transition) {
		super(LABEL);

		// Only continue if we have one incoming/outgoing...
		if (transition.getIncoming().size() != 1)
			return;
		if (transition.getOutgoing().size() != 1)
			return;

		// Get connected HLPlaces
		HLPlace sourcePlace = (HLPlace) ((HLArc) (transition.getIncoming()
				.get(0))).getSource();
		HLPlace targetPlace = (HLPlace) ((HLArc) (transition.getOutgoing()
				.get(0))).getTarget();

		// Only continue if we have NET HLPlaces
		if (sourcePlace == null
				|| sourcePlace.getType().getValue() != HLPlaceType.NET)
			return;
		if (targetPlace == null
				|| targetPlace.getType().getValue() != HLPlaceType.NET)
			return;

		if (sourcePlace.getTokens().size() > 0) {
			ObjectNet token = (ObjectNet) sourcePlace.getTokens().get(0);
			add(new SplitObjectNetCommand(token, targetPlace));
		}
	}
}
