package roneditor.gef.commands.ron;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.gef.commands.CompoundCommand;

import roneditor.model.RONModelManager;
import ronmodel.HLArc;
import ronmodel.HLPlace;
import ronmodel.HLPlaceType;
import ronmodel.HLTransition;
import ronmodel.ObjectNet;
import ronmodel.Token;

/**
 * Perform <code>STANDARD</code> operation on the {@link HLTransition}.
 * <ul>
 * <li>We copy the contents of <i>all</i> {@link Token} on incoming
 * {@link HLPlace}s from the {@link HLPlaceType#NET} into the same number of
 * {@link ObjectNet}s as we have outgoing {@link HLPlace} of the
 * {@link HLPlaceType#NET}, put the new {@link Token}s onto the targets and
 * remove the original {@link Token}s from the source.</li>
 * 
 * <li>As we require to have at most <i>one</i> {@link HLPlace} of
 * {@link HLPlaceType#RULE}, we just <i>move</i> the {@link Token} to the
 * target.</li>
 * </ul>
 */
public class HLTransitionStandardCommand extends CompoundCommand {

	
	@SuppressWarnings("unchecked")
	public HLTransitionStandardCommand(HLTransition transition) {

		// Collect incoming NET/RULE HLPlaces
		List<HLArc> incomingArc = transition.getIncoming();
		List<HLPlace> incomingNetPlaces = new LinkedList<HLPlace>();
		List<HLPlace> incomingRulePlaces = new LinkedList<HLPlace>();
		for (HLArc arc : incomingArc) {
			HLPlace place = (HLPlace) arc.getSource();
			// no token on place? -> return
			if (place == null) return;
			if (place.getTokens().size() == 0) return;
			if (place.getType().getValue() == HLPlaceType.NET)
				incomingNetPlaces.add(place);
			else
				incomingRulePlaces.add(place);
		}

		// Do not execute with more than on RULE
		if (incomingRulePlaces.size() > 1)
			return;

		// Collect outgoing NET/RULE HLPlaces
		List<HLArc> outgoingArc = transition.getOutgoing();
		List<HLPlace> outgoingNetPlaces = new LinkedList<HLPlace>();
		List<HLPlace> outgoingRulePlaces = new LinkedList<HLPlace>();
		for (HLArc arc : outgoingArc) {
			HLPlace place = (HLPlace) arc.getTarget();
			if (place == null) return;
			if (place.getType().getValue() == HLPlaceType.NET)
				outgoingNetPlaces.add(place);
			else
				outgoingRulePlaces.add(place);
		}

		 	
		if (incomingRulePlaces.size() == 0 && outgoingRulePlaces.size() > 0) return;
		if (incomingNetPlaces.size() == 0 && outgoingNetPlaces.size() > 0) return;
		
		//process NetPlaces
		if (incomingNetPlaces.size() > 0){
			
			//Collect all NET tokens to move/combine
			//also consider multiple tokens from the same place
			List<ObjectNet> tokens = new LinkedList<ObjectNet>();
			HashMap<HLPlace, Integer> tokenizer = new HashMap<HLPlace, Integer>();
			for (HLPlace p : incomingNetPlaces) {
				List<ObjectNet> nets = p.getTokens();
				Integer neu = null;
				Integer counter = tokenizer.get(p);
				if (counter == null)
					neu = new Integer(0);
				else
					neu = new Integer(counter.intValue()+1);
				tokenizer.put(p, neu);
				tokens.add( nets.get(neu.intValue()) );	
				
			}
			
			//union of all first token of incomingNetPlaces into one unionObjectNet
			ObjectNet unionObjectNet = RONModelManager.getFactory().createObjectNet();
			add(new UnionObjectNetsIntoOneCommand(tokens,unionObjectNet) );

			
			// Copy unionObjectNet to the new outgoing tokens on HLPlace.NET
			for (HLPlace place : outgoingNetPlaces) {
				add(new CopyTokenToHLPlaceCommand((Token)unionObjectNet, place));
			}
			
			// remove tokens/ObjectNets from place
			for (ObjectNet t : tokens) {
				add(new RemoveTokenFromHLPlaceCommand(t));
			}

		}

		//process RulePlaces
		if(incomingRulePlaces.size() > 0 ){
			HLPlace incomingRulePlace = incomingRulePlaces.get(0);
			Token token = (Token)incomingRulePlace.getTokens().get(0);

			// Copy rule to the new outgoing tokens on HLPlace.RULE
			for (HLPlace place : outgoingRulePlaces) {
				add(new CopyTokenToHLPlaceCommand((Token)token, place));
			}
			
			// remove rule from the HLPlace of Type RULE
			add(new RemoveTokenFromHLPlaceCommand(token) );
		}

	}

}
