package roneditor.gef.commands.rule;

import java.util.HashMap;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.impl.NotificationImpl;
import org.eclipse.gef.commands.Command;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.PlatformUI;

import roneditor.model.RuleChecker;
import roneditor.model.RuleSimulator;
import ronmodel.ONNode;
import ronmodel.ObjectNet;
import ronmodel.Rule;
import ronmodel.Token;

/**
 * Apply a single {@link Rule} to an single {@link ObjectNet}
 */
public class ApplyRuleCommand extends Command {
	public static final String LABEL = "ApplyRuleCommand";

	protected Token rule;

	protected Token objectNet;

	protected RuleSimulator ruleSimulator;
	
	protected boolean success;
	
	protected HashMap<ONNode, ONNode> match;
	
	protected String lastErrorMessage = "No match";

	public ApplyRuleCommand(Token objectNet, Token rule) {
		super(LABEL);
		this.objectNet = objectNet;
		this.rule = rule;
		this.match = null;
		this.ruleSimulator = new RuleSimulator();
	}

	public ApplyRuleCommand(Token objectNet, Token rule, HashMap<ONNode, ONNode> match) {
		this(objectNet, rule);
		this.match = match;
	}


	
	
	/**
	 * Check if the {@link Rule} can be applied to the {@link ObjectNet}.
	 * 
	 * @return true if the {@link RuleSimulator} can apply the rule.
	 */
	public boolean canExecute() {
		boolean typeOK = (rule instanceof Rule)
				&& (objectNet instanceof ObjectNet);
		if (!typeOK)
			return false;

		// check if rule is really ok
		Rule r = (Rule) rule;
		ObjectNet o = (ObjectNet) objectNet;
		IEditorPart e = PlatformUI.getWorkbench().getActiveWorkbenchWindow()
				.getActivePage().getActiveEditor();

		if (!roneditor.model.RuleChecker.isRuleOk(e, r))
			return false;

		// check if all nodes in the objectnet have a name

		RuleChecker.editor = e;
		// RuleChecker.deleteAllMarkers();
		for (Object n : o.getNodes()) {
			if (((ONNode) n).getName().compareTo("") == 0) {
				RuleChecker.createErrorMarker(o.getName(),
						"There are nodes with empty names.");
				return false;
			}
		}
		
		boolean isApplicable;
		if ( match != null )
			isApplicable = ruleSimulator.isApplicable(match, (Rule) rule,
				(ObjectNet) objectNet);
		else
			isApplicable = ruleSimulator.isApplicable((Rule) rule,
					(ObjectNet) objectNet);
			
		setLastErrorMessage(ruleSimulator.getLastMessage());
		return isApplicable;
	}

	/**
	 * Apply the {@link Rule} to the {@link ObjectNet}
	 * 
	 * @see RuleSimulator#applyRule(Rule, ObjectNet)
	 */
	public void execute() {
		if ( match != null )
			success = ruleSimulator.applyRule(match, (Rule) rule, (ObjectNet) objectNet);
		else
			success = ruleSimulator.applyRule((Rule) rule, (ObjectNet) objectNet);
			
		Notification notification = new NotificationImpl(0, true, false);
	
		// Wake up all children.
		for (Object o : ((ObjectNet) objectNet).getNodes()) {
			ONNode node = (ONNode) o;
			node.eNotify(notification);
		}
	}

	public boolean wasSuccessful() {
		return success;
	}
	
	public void undo() {
		ruleSimulator.undo();
	}

	public void redo() {
		ruleSimulator.redo();
	}

	public String getLastErrorMessage() {
		return lastErrorMessage;
	}

	public void setLastErrorMessage(String lastErrorMessage) {
		this.lastErrorMessage = lastErrorMessage;
	}

}
