package roneditor.gef.editparts.ron;

import java.util.List;

import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.NodeEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.ui.views.properties.IPropertySource;

import roneditor.gef.editpolicies.ron.HLNodeComponentEditPolicy;
import roneditor.gef.editpolicies.ron.HLNodeDirectEditPolicy;
import roneditor.gef.figures.objectnet.NodeFigure;
import roneditor.ui.views.properties.EObjectPropertySource;
import ronmodel.HLNode;

/**
 * EditPart for a generic HLNode. The derived {@link HLTransitionEditPart} and
 * {@linkHL PlaceEditPolicy} need to implement the deviant functionality.
 */
abstract public class HLNodeEditPart extends AbstractGraphicalEditPart implements NodeEditPart, Adapter {
	protected IPropertySource propertySource = null;

	protected Notifier target;

	/**
	 * Creates a new {@link HLNodeEditPart} instance.
	 * 
	 * @param node
	 *            The Node
	 */
	public HLNodeEditPart(HLNode node) {
		setModel(node);
	}

	/**
	 * Get the model as {@link HLNode}
	 */
	public HLNode getModelAsHLNode() {
		return (HLNode) getModel();
	}

	
	
	/**
	 * Get the incoming connections of the node
	 * 
	 * @return List of HLNodes
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelSourceConnections()
	 */
	protected List getModelSourceConnections() {
		return getModelAsHLNode().getIncoming();
	}

	/**
	 * Get the incoming connections of the node
	 * 
	 * @return List of HLNodes
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelTargetConnections()
	 */
	protected List getModelTargetConnections() {
		return getModelAsHLNode().getOutgoing();
	}

	/**
	 * Executes the passed command.
	 * 
	 * @param command
	 *            the command to execute
	 */
	protected void executeCommand(Command command) {
		getEditDomain().getCommandStack().execute(command);
	}

	/**
	 * Convenience method returning the editpart's edit domain. Same as calling
	 * <code>getRoot().getViewer().getEditDomain()</code>
	 * 
	 * @return the edit domain
	 */
	protected EditDomain getEditDomain() {
		return getRoot().getViewer().getEditDomain();
	}

	/**
	 * Perform a DirectEdit (single click) via a request.
	 */
	abstract protected void performDirectEdit();

	/**
	 * Perform an Open request (double click). Currently calls
	 * performDirectEdit()
	 */
	abstract protected void performOpen();

	/**
	 * Handles request such as clicks on the EditPart.
	 */
	public void performRequest(Request request) {
		if (request.getType() == RequestConstants.REQ_DIRECT_EDIT) // single
			// click
			performDirectEdit();
		else if (request.getType() == RequestConstants.REQ_OPEN) // double
			// click
			performOpen();
	}

	/**
	 * Returns the Figure of this, as a transition type figure.
	 * 
	 * @return Figure as a TransitionFigure.
	 */
	protected NodeFigure getNodeFigure() {
		return (NodeFigure) getFigure();
	}

	/**
	 * Create all basic policies for a ONNode.
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		// install the edit policy to handle deletion
		installEditPolicy(EditPolicy.COMPONENT_ROLE, new HLNodeComponentEditPolicy());

		// install the edit policy for direct editing of the label
		installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new HLNodeDirectEditPolicy());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
	 */
	public ConnectionAnchor getSourceConnectionAnchor(ConnectionEditPart connection) {
		return getNodeFigure().getSourceConnectionAnchor();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.Request)
	 */
	public ConnectionAnchor getSourceConnectionAnchor(Request request) {
		return getNodeFigure().getSourceConnectionAnchor();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
	 */
	public ConnectionAnchor getTargetConnectionAnchor(ConnectionEditPart connection) {
		return getNodeFigure().getTargetConnectionAnchor();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.Request)
	 */
	public ConnectionAnchor getTargetConnectionAnchor(Request request) {
		return getNodeFigure().getTargetConnectionAnchor();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */
	protected void refreshVisuals() {
		getNodeFigure().setRuleMode(false);
		getNodeFigure().setName(getModelAsHLNode().getName());
		Point loc = new Point(getModelAsHLNode().getX(), getModelAsHLNode().getY());
		Dimension size = new Dimension(-1, -1);
		Rectangle r = new Rectangle(loc, size);
		((GraphicalEditPart) getParent()).setLayoutConstraint(this, getFigure(), r);
		getNodeFigure().validate();
		super.refreshVisuals();
	}

	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#activate()
	 */
	@SuppressWarnings("unchecked")
	public void activate() {
		if (isActive())
			return;

		// start listening for changes in the model
		((Notifier) getModel()).eAdapters().add(this);

		super.activate();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#deactivate()
	 */
	public void deactivate() {
		if (!isActive())
			return;

		// stop listening for changes in the model
		((Notifier) getModel()).eAdapters().remove(this);

		super.deactivate();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
	 */
	public Notifier getTarget() {
		return target;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.emf.common.notify.Adapter#isAdapterForType(java.lang.Object)
	 */
	public boolean isAdapterForType(Object type) {
		return type.equals(getModel().getClass());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
	 */
	public void setTarget(Notifier newTarget) {
		target = newTarget;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	public Object getAdapter(Class key) {
		if (IPropertySource.class == key) {
			return getPropertySource();
		}
		return super.getAdapter(key);
	}
	

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.itso.sal330r.gefdemo.edit.ObjectNetElementEditPart#getPropertySource()
	 */
	protected IPropertySource getPropertySource() {
		if (propertySource == null) {
			propertySource = new EObjectPropertySource(getModelAsHLNode());
		}
		return propertySource;
	}

}
