package roneditor.gef.editparts.ron;

import java.util.List;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.NodeEditPart;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.views.properties.IPropertySource;

import roneditor.gef.commands.ron.HLTransitionApplyRuleCommand;
import roneditor.gef.commands.ron.HLTransitionFireCommand;
import roneditor.gef.commands.ron.HLTransitionSplitCommand;
import roneditor.gef.commands.ron.HLTransitionStandardCommand;
import roneditor.gef.editpolicies.ron.HLTransitionEditPolicy;
import roneditor.gef.figures.objectnet.TransitionFigure;
import roneditor.gef.tools.objectnet.ONTransitionCellEditorLocator;
import roneditor.gef.tools.ron.NodeEditManager;
import roneditor.model.HLTransitionUtil;
import roneditor.model.ONTransitionUtil;
import roneditor.ui.utils.RONIconUtil;
import roneditor.ui.views.properties.HLTransitionPropertySource;
import roneditor.ui.views.properties.ONTransitionPropertySource;
import ronmodel.HLArc;
import ronmodel.HLPlace;
import ronmodel.HLPlaceType;
import ronmodel.HLTransition;
import ronmodel.HLTransitionKind;
import ronmodel.ONNode;
import ronmodel.ONPlace;
import ronmodel.ONTransition;
import ronmodel.ObjectNet;
import ronmodel.RonmodelPackage;

/**
 * EditPart for an HLTransition. TODO: Write more stuff here!
 */
public class HLTransitionEditPart extends HLNodeEditPart implements NodeEditPart, Adapter {

	public HLTransitionEditPart(HLTransition server) {
		super(server);
	}

	/**
	 * This method is executed on an double click. Usability wise we normally
	 * fall back to direct edit but the transition should fire on a
	 * double-click.
	 * 
	 */
	protected void performOpen() {
		switch (getCastedModel().getKind().getValue()) {
		case HLTransitionKind.FIRE:
			executeCommand(new HLTransitionFireCommand(getCastedModel()));
			break;
		case HLTransitionKind.APPLYRULE:
			HLTransitionApplyRuleCommand command = new HLTransitionApplyRuleCommand(getCastedModel());
			if (!command.canExecute()) {
				IWorkbench workbench = PlatformUI.getWorkbench();
				Shell shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();
				Status status = new Status(IStatus.ERROR, "RONEditor", 0, command.getLastErrorMessage(), null);
				ErrorDialog.openError(shell, "APPLYRULE", "Could not apply the rule", status);
			}
			executeCommand(command);
			break;
		case HLTransitionKind.SPLIT:
			executeCommand(new HLTransitionSplitCommand(getCastedModel()));
			break;
		case HLTransitionKind.STANDARD:
			executeCommand(new HLTransitionStandardCommand(getCastedModel()));
			break;
		default:
			break;
		}
	}

	/**
	 * Create the edit policies for the Transition
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		super.createEditPolicies();

		// install the edit policy to handle connection creation
		installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE, new HLTransitionEditPolicy());
	}

	/**
	 * Update the colour of the figure depending if the transition can fire.
	 * Then execute super method.
	 * 
	 * @see ron.HLNodeEditPart#refreshVisuals()
	 */
	protected void refreshVisuals() {
		((TransitionFigure) getFigure()).setIcon(RONIconUtil.getInstance().getIcon(RONIconUtil.HL_TRANSITION,
				getCastedModel().getKind().getValue(), 16));

		Color c;
		if (!HLTransitionUtil.isValid(getCastedModel())) {
			// mark invalid transition
			c = new Color(null, 250, 100, 100);
		} else {
			// gray out transition that can not fire
			if (HLTransitionUtil.canFire(getCastedModel()))
				switch (getCastedModel().getKind().getValue()) {
				case HLTransitionKind.APPLYRULE:
					c = new Color(null, 65, 255, 82);
					break;
				case HLTransitionKind.FIRE:
					c = new Color(null, 255, 226, 90);
					break;
				case HLTransitionKind.SPLIT:
					c = new Color(null, 180, 220, 255);
					break;
				case HLTransitionKind.STANDARD:
					c = new Color(null, 179, 126, 238);
					break;
				default:
					c = new Color(null, 250, 100, 100);
				}
			else
				c = new Color(null, 220, 220, 220);
		}
		((TransitionFigure) getFigure()).setColor(c);
		super.refreshVisuals();
	}



	/**
	 * Handle notifications like changes of connections. Defaults to
	 * refreshVisuals().
	 */
	public void notifyChanged(Notification notification) {
		int featureId = notification.getFeatureID(RonmodelPackage.class);
		switch (featureId) {
		case RonmodelPackage.HL_NODE__OUTGOING:
			refreshTargetConnections();
			refreshVisuals();
			break;
		case RonmodelPackage.HL_NODE__INCOMING:
			refreshSourceConnections();
			refreshVisuals();
			break;
		default:
			refreshVisuals();
			break;
		}
	}

	/**
	 * Return the current model casted as HLTranstition.
	 * 
	 * @return the current model as HLTransition
	 */
	public HLTransition getCastedModel() {
		return (HLTransition) getModel();
	}


	/**
	 * Create the TransitionFigure as figure for the EditPart.
	 * 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure() {
		TransitionFigure figure = new TransitionFigure();
		return figure;
	}

	/**
	 * Perform the direct edit on the EditPart. Shows an {@link NodeEditManager}
	 * whose location is controlled by the {@link ONTransitionCellEditorLocator}.
	 */
	protected void performDirectEdit() {
		new NodeEditManager(this, new ONTransitionCellEditorLocator((TransitionFigure) getFigure())).show();
	}

	/**
	 * Return the appropriate {@link IPropertySource}, namely
	 * {@link ONTransitionPropertySource} for the PropertyView
	 * 
	 * return The {@link IPropertySource} for the current Model
	 */
	protected IPropertySource getPropertySource() {
		if (propertySource == null) {
			propertySource = new HLTransitionPropertySource((HLTransition) getModel());
		}
		return propertySource;
	}

}
