package roneditor.gef.editparts.tree;

import java.util.List;

import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.gef.DragTracker;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editparts.AbstractTreeEditPart;
import org.eclipse.gef.tools.DragTreeItemsTracker;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.views.properties.IPropertySource;

import roneditor.gef.actions.ron.CreateTokenAction;
import roneditor.gef.commands.ron.CreateTokenCommand;
import roneditor.gef.editpolicies.NamedElementDirectEditPolicy;
import roneditor.gef.editpolicies.ron.HLNodeComponentEditPolicy;
import roneditor.gef.editpolicies.ron.HLPlaceLayoutEditPolicy;
import roneditor.gef.tools.tree.TreeDirectEditManager;
import roneditor.model.RONModelManager;
import roneditor.ui.utils.RONIconUtil;
import roneditor.ui.views.properties.HLNodePropertySource;
import roneditor.ui.views.properties.HLPlacePropertySource;
import ronmodel.HLPlace;
import ronmodel.HLPlaceType;
import ronmodel.ObjectNet;
import ronmodel.RonmodelPackage;
import ronmodel.Rule;

/**
 * TreeEditPart for HLPlaces.
 * 
 * @author Tony Modica
 */
public class HLPlaceTreeEditPart extends AbstractTreeEditPart implements Adapter {

	/**
	 * Target field for {@link Adapter} implementation.
	 */
	private Notifier target;

	private TreeDirectEditManager manager;

	private IPropertySource propertySource;

	public HLPlaceTreeEditPart(final Object model) {
		super(model);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractTreeEditPart#createEditPolicies()
	 */
	@Override
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.COMPONENT_ROLE, new HLNodeComponentEditPolicy());
		installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new NamedElementDirectEditPolicy());
		installEditPolicy(EditPolicy.LAYOUT_ROLE, new HLPlaceLayoutEditPolicy());
	}

	/**
	 * Returns a List of all Tokens on the HLPlace which is the model of this
	 * EditPart.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#getModelChildren()
	 */
	@SuppressWarnings("unchecked")
	@Override
	protected List<Object> getModelChildren() {
		return getCastedModel().getTokens();
	}

	/**
	 * Convenience method that casts the model to the appropriate type.
	 * 
	 * @return The model of this EditPart appropriately casted.
	 */
	public HLPlace getCastedModel() {
		return (HLPlace) getModel();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractTreeEditPart#getImage()
	 */
	@Override
	protected Image getImage() {
		return RONIconUtil.getInstance().getIcon(RonmodelPackage.HL_PLACE, getCastedModel().getType().getValue(), 16);
	}

	/**
	 * Besides the requests handled by the installed EditPolicies (none for this
	 * EditPart) this EditPart understands following request.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.EditPart#understandsRequest(org.eclipse.gef.Request)
	 */
	@Override
	public boolean understandsRequest(final Request req) {
		final Object type = req.getType();
		return type.equals(CreateTokenAction.ID) || super.understandsRequest(req);
	}

	/**
	 * Returns a Command if the appropriate request is passed. Checks the
	 * Commands handled by installed EditPolicies (none for this EditPart) as
	 * well.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.EditPart#getCommand(org.eclipse.gef.Request)
	 */
	@SuppressWarnings("unchecked")
	@Override
	public Command getCommand(final Request request) {
		if (request.getType().equals(CreateTokenAction.ID)) {
			if (getCastedModel().getType().equals(HLPlaceType.NET_LITERAL)) {
				final ObjectNet newToken = RONModelManager.getFactory().createObjectNet();
				return new CreateTokenCommand(getCastedModel(), newToken);
			} else if (getCastedModel().getType().equals(HLPlaceType.RULE_LITERAL)) {
				final Rule newToken = RONModelManager.getFactory().createRule();
				newToken.setLhs(RONModelManager.getFactory().createLHS());
				newToken.setRhs(RONModelManager.getFactory().createRHS());
				newToken.getNacs().add(RONModelManager.getFactory().createNAC());
				return new CreateTokenCommand(getCastedModel(), newToken);
			}
		}
		return super.getCommand(request);
	}

	/**
	 * Hooks in this EditPart as a listener for changes of its model.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.EditPart#activate()
	 */
	@SuppressWarnings("unchecked")
	@Override
	public void activate() {
		super.activate();
		getCastedModel().eAdapters().add(this);
	}

	/**
	 * Hooks off this EditPart as a listener for changes of its model.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.EditPart#deactivate()
	 */
	@Override
	public void deactivate() {
		super.deactivate();
		getCastedModel().eAdapters().remove(this);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractTreeEditPart#getText()
	 */
	@Override
	protected String getText() {
		return getCastedModel().getName();
	}

	/**
	 * Returns a DragTracker to handle direct editing and/or opening.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractTreeEditPart#getDragTracker(org.eclipse.gef.Request)
	 */
	@Override
	public DragTracker getDragTracker(final Request req) {
		return new DragTreeItemsTracker(this);
	}

	/**
	 * This handles the direct edit and open request sent by the SelectionTool.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.EditPart#performRequest(org.eclipse.gef.Request)
	 */
	@Override
	public void performRequest(final Request req) {
		if (req.getType().equals(RequestConstants.REQ_DIRECT_EDIT)) {
			performDirectEdit();
		}
	}

	/**
	 * Uses a TreeDirectEditManager to perform a direct edit on this EditPart.
	 */
	private void performDirectEdit() {
		if (checkTreeItem() && manager == null) {
			manager = new TreeDirectEditManager(this);
		}
		manager.show();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#getAdapter(java.lang.Class)
	 */
	@Override
	public Object getAdapter(final Class key) {
		if (IPropertySource.class == key) {
			return getPropertySource();
		}
		return super.getAdapter(key);
	}

	/**
	 * @return A HLNodePropertySource for the Properties View.
	 */
	private IPropertySource getPropertySource() {
		if (propertySource == null) {
			propertySource = new HLPlacePropertySource(getCastedModel());
		}
		return propertySource;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.emf.common.notify.Adapter#isAdapterForType(java.lang.Object)
	 */
	public boolean isAdapterForType(final Object type) {
		return type.equals(getModel().getClass());
	}

	/**
	 * If the model fires a property change notification to its listeners, the
	 * visual elements which are managed by this EditPart will be refrehed
	 * depending on the featureID the notifiation contains.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.emf.common.notify.Adapter#notifyChanged(org.eclipse.emf.common.notify.Notification)
	 */
	public void notifyChanged(final Notification notification) {
		final int featureId = notification.getFeatureID(RonmodelPackage.class);
		switch (featureId) {
		case RonmodelPackage.HL_PLACE__TOKENS:
			refreshChildren();
			break;
		case RonmodelPackage.HL_PLACE__NAME:
			refreshVisuals();
			break;
		default:
			break;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
	 */
	public void setTarget(final Notifier newTarget) {
		target = newTarget;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
	 */
	public Notifier getTarget() {
		return target;
	}
}
