package roneditor.gef.editpolicies.objectnet;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gef.requests.ReconnectRequest;

import roneditor.gef.commands.rule.UpdateMappingCommand;
import roneditor.gef.editparts.objectnet.ONNodeEditPart;
import roneditor.gef.figures.objectnet.NodeFigure;
import ronmodel.Codomain;
import ronmodel.LHS;
import ronmodel.Mapping;
import ronmodel.ONNode;

/**
 * {@link GraphicalNodeEditPolicy} for the {@link ONNodeEditPart}. Provides
 * some convenience functions and initiates the feedback and connection handling
 * of {@link Mapping} commands.
 */
public class ONNodeEditPolicy extends GraphicalNodeEditPolicy {

	public ONNodeEditPolicy() {
		super();
	}

	/**
	 * Return the casted model of the host {@link EditPart}.
	 * 
	 * @return
	 */
	protected ONNode getCastedModel() {
		return (ONNode) getHost().getModel();
	}

	/**
	 * Return the {@link EditPartViewer} of the host {@link EditPart}.
	 * 
	 * @return
	 */
	public EditPartViewer getViewer() {
		return getHost().getRoot().getViewer();
	}

	/**
	 * Return the {@link Codomain} of the {@link EditPartViewer}.
	 * 
	 * @return
	 */
	public Codomain getCodomain() {
		Object model = getViewer().getContents().getModel();
		if (model instanceof Codomain)
			return (Codomain) model;
		return null;
	}

	/**
	 * If we create a {@link Mapping} we want to have another feedback. This
	 * method is called while the connection is not finished.
	 */
	@Override
	public void showSourceFeedback(Request request) {
		if (request instanceof CreateConnectionRequest) {
			CreateConnectionRequest createConnectionRequest = (CreateConnectionRequest) request;
			Object requestingObject = createConnectionRequest.getNewObject();
			if (requestingObject instanceof Mapping) {
				if ( !(getCastedModel().getObjectNet() instanceof LHS) ) return;
				((NodeFigure) getHostFigure()).setGlow(true);

				return;
			}
		}
		super.showSourceFeedback(request);
	}

	/**
	 * If we have create a {@link Mapping} we want to have another feedback.
	 * This method is called when the connection is finished.
	 */
	@Override
	public void eraseSourceFeedback(Request request) {
		if (request instanceof CreateConnectionRequest) {
			CreateConnectionRequest createConnectionRequest = (CreateConnectionRequest) request;
			Object requestingObject = createConnectionRequest.getNewObject();
			if (requestingObject instanceof Mapping) {
				if ( !(getCastedModel().getObjectNet() instanceof LHS) ) return;
				((NodeFigure) getHostFigure()).setGlow(false);
				return;
			}
		}
		super.eraseSourceFeedback(request);
	}

	/**
	 * Called whenever the {@link CreateConnectionRequest} requests the
	 * completion {@link Command}. When our start {@link Command} is an
	 * {@link UpdateMappingCommand} we update the {@link Codomain} and target
	 * model appropriately.
	 */
	@Override
	protected Command getConnectionCompleteCommand(CreateConnectionRequest request) {
		Command command = request.getStartCommand();
		if (command instanceof UpdateMappingCommand) {
			UpdateMappingCommand createMappingCommand = (UpdateMappingCommand) command;
			createMappingCommand.setCodomain(getCodomain());
			createMappingCommand.setTarget(getCastedModel());
			//createMappingCommand.setTargetViewer(getViewer());
			createMappingCommand.update();
			return command;
		}
		return null;
	}

	/**
	 * Called whenever the {@link CreateConnectionRequest} requests the creation
	 * {@link Command}. Handles the creation of the
	 * {@link UpdateMappingCommand} when we want to create a {@link Mapping}.
	 */
	@Override
	protected Command getConnectionCreateCommand(CreateConnectionRequest request) {
		Object requestingObject = request.getNewObject();
		if (requestingObject instanceof Mapping) {
			if ( !(getCastedModel().getObjectNet() instanceof LHS) ) return null;
			UpdateMappingCommand command = (UpdateMappingCommand) request.getStartCommand();
			if (command == null) {
				command = new UpdateMappingCommand();
				command.setMapping((Mapping) requestingObject);
			}
			command.setSource(getCastedModel());
			command.update();
			//command.setSourceViewer(getViewer());
			request.setStartCommand(command);
			return command;
		}
		return null;
	}

	@Override
	protected Command getReconnectSourceCommand(ReconnectRequest request) {
		return null;
	}

	@Override
	protected Command getReconnectTargetCommand(ReconnectRequest request) {
		return null;
	}

}
