package roneditor.gef.editpolicies.objectnet;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gef.requests.ReconnectRequest;

import roneditor.gef.commands.objectnet.ConnectPlaceTransitionCommand;
import roneditor.gef.commands.objectnet.ConnectTransitionPlaceCommand;
import roneditor.gef.commands.objectnet.FireTransitionCommand;
import roneditor.gef.commands.objectnet.ReconnectCommand;
import roneditor.gef.editparts.objectnet.ONTransitionEditPart;
import ronmodel.ONArc;
import ronmodel.ONNode;
import ronmodel.ONPlace;
import ronmodel.ONTransition;

/**
 * {@link EditPolicy} for an {@link ONTransitionEditPart}.
 */
public class ONTransitionEditPolicy extends ONNodeEditPolicy {

	public static String FIRE_REQUEST = "ONTransitionEditPolicyFire";

	public ONTransitionEditPolicy() {
		super();
	}

	public Command getCommand(final Request request) {
		if (request.getType() == ONTransitionEditPolicy.FIRE_REQUEST) {
			return getFireCommand();
		}

		return super.getCommand(request);
	}

	/**
	 * @return An FireTransitionCommand with the model of the host EditPart set.
	 */
	private Command getFireCommand() {
		return new FireTransitionCommand((ONTransition) getHost().getModel());
	}

	/**
	 * When ever a connection is completed, we update the provided
	 * {@link CreateConnectionRequest} with the target, hence the
	 * {@link ONTransition} of the {@link ONTransitionEditPart}.
	 * 
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCompleteCommand(org.eclipse.gef.requests.CreateConnectionRequest)
	 */
	protected Command getConnectionCompleteCommand(CreateConnectionRequest request) {
		Command command = request.getStartCommand();
		if (command instanceof ConnectPlaceTransitionCommand) {
			ConnectPlaceTransitionCommand commandSC = (ConnectPlaceTransitionCommand) command;
			commandSC.setTarget(getCastedModel());
			return command;
		}
		return super.getConnectionCompleteCommand(request);
	}

	public ONTransition getCastedModel() {
		return (ONTransition) super.getCastedModel();
	}

	/**
	 * When ever a connection is created, we update the provided
	 * {@link CreateConnectionRequest} with the source, hence the
	 * {@link ONTransition} of the {@link ONTransitionEditPart}.
	 * 
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCreateCommand(org.eclipse.gef.requests.CreateConnectionRequest)
	 */
	protected Command getConnectionCreateCommand(CreateConnectionRequest request) {
		Object requestingObject = request.getNewObject();
		if (requestingObject instanceof ONArc) {
			ConnectTransitionPlaceCommand command = new ConnectTransitionPlaceCommand();
			request.setStartCommand(command);
			command.setArc((ONArc) requestingObject);
			command.setSource(getCastedModel());
			return command;
		}
		return super.getConnectionCreateCommand(request);
	}

	/**
	 * When ever a connection is changed, we update the provided
	 * {@link ReconnectRequest} with the target, hence the {@link ONTransition}
	 * of the {@link ONTransitionEditPart}.
	 * 
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCreateCommand(org.eclipse.gef.requests.CreateConnectionRequest)
	 */
	protected Command getReconnectTargetCommand(ReconnectRequest request) {
		ReconnectCommand command = new ReconnectCommand();
		ONArc link = (ONArc) request.getConnectionEditPart().getModel();
		EditPart target = request.getTarget();
		if (link.getTarget() instanceof ONPlace && link.getSource() != target.getModel()) {
			command.setArc(link);
			command.setSource((ONNode) target.getModel());
			command.setTarget(link.getTarget());
			return command;
		} else
			return null;
	}

	/**
	 * When ever a connection is changed, we update the provided
	 * {@link ReconnectRequest} with the source, hence the {@link ONTransition}
	 * of the {@link ONTransitionEditPart}.
	 * 
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCreateCommand(org.eclipse.gef.requests.CreateConnectionRequest)
	 */
	protected Command getReconnectSourceCommand(ReconnectRequest request) {
		ReconnectCommand command = new ReconnectCommand();
		ONArc link = (ONArc) request.getConnectionEditPart().getModel();
		EditPart target = request.getTarget();
		if (link.getSource() instanceof ONPlace && link.getTarget() != target.getModel()) {
			command.setArc(link);
			command.setSource(link.getSource());
			command.setTarget((ONNode) target.getModel());
			return command;
		} else
			return null;
	}
}
