package roneditor.gef.tools.tree;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartListener;
import org.eclipse.gef.TreeEditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.gef.requests.DirectEditRequest;
import org.eclipse.gef.tools.DirectEditManager;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ICellEditorListener;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.swt.events.VerifyListener;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;

import ronmodel.NamedElement;

/**
 * A helper class to perform a direct edit on a TreeEditPart with a
 * TextCellEditor. Adapted for {@link TreeEditPart}s from
 * {@link DirectEditManager}.
 * 
 * @author Tony Modica
 */
public class TreeDirectEditManager {

	private ICellEditorListener cellEditorListener;

	/**
	 * Is <code>true</code> if the cell editor's value has been changed.
	 */
	private boolean dirty;

	private DirectEditRequest request;

	private EditPartListener editPartListener;

	private TreeEditPart source;

	private CellEditor cellEditor;

	private boolean committing = false;

	private org.eclipse.swt.custom.TreeEditor tableEditor;

	/**
	 * Constructs a new DirectEditManager for the given source edit part. A new
	 * TextCellEditor will be created and placed using the given
	 * CellEditorLocator.
	 * 
	 * @param source
	 *            The source edit part
	 */
	public TreeDirectEditManager(final TreeEditPart source) {
		this.source = source;
	}

	/**
	 * Cleanup is done here. Any feedback is erased and listeners unhooked. If
	 * the cell editor is not <code>null</code>, it will be
	 * {@link CellEditor#deactivate() deativated},
	 * {@link CellEditor#dispose() disposed}, and set to <code>null</code>.
	 */
	public void bringDown() {
		unhookListeners();
		if (cellEditor != null) {
			cellEditor.deactivate();
			cellEditor.dispose();
			setCellEditor(null);
		}
		if (tableEditor != null) {
			tableEditor.getEditor().dispose();
			tableEditor.dispose();
			tableEditor = null;
		}
		request = null;
		dirty = false;
	}

	/**
	 * Shows the cell editor when direct edit is started. Calls
	 * {@link #initCellEditor()} and {@link CellEditor#activate()}.
	 */
	public void show() {
		if (cellEditor != null) {
			return;
		}
		final Composite composite = ((TreeItem) source.getWidget()).getParent();
		setCellEditor(new TextCellEditor(composite));
		if (cellEditor == null) {
			return;
		}
		tableEditor = new org.eclipse.swt.custom.TreeEditor((Tree) composite);

		final CellEditor.LayoutData layout = cellEditor.getLayoutData();
		tableEditor.horizontalAlignment = layout.horizontalAlignment;
		tableEditor.grabHorizontal = layout.grabHorizontal;
		tableEditor.minimumWidth = layout.minimumWidth;
		tableEditor.setEditor(cellEditor.getControl(), (TreeItem) source.getWidget(), 0);

		initCellEditor();
		cellEditor.activate();
		cellEditor.setFocus();
	}

	/**
	 * Commits the current value of the cell editor by getting a {@link Command}
	 * from the source edit part and executing it via the {@link CommandStack}.
	 * Finally, {@link #bringDown()} is called to perform and necessary cleanup.
	 */
	private void commit() {
		if (committing) {
			return;
		}
		committing = true;
		try {
			if (dirty) {
				final CommandStack stack = source.getViewer().getEditDomain().getCommandStack();
				stack.execute(source.getCommand(getDirectEditRequest()));
			}
		} finally {
			bringDown();
			committing = false;
		}
	}

	private DirectEditRequest getDirectEditRequest() {
		if (request == null) {
			request = new DirectEditRequest();
		}
		return request;
	}

	private void hookListeners() {
		cellEditorListener = new ICellEditorListener() {

			public void applyEditorValue() {
				getDirectEditRequest().setDirectEditFeature(cellEditor.getValue());
				commit();
			}

			public void cancelEditor() {
				bringDown();
			}

			public void editorValueChanged(@SuppressWarnings("unused")
			final boolean old, final boolean newState) {
				dirty = newState;
			}
		};
		cellEditor.addListener(cellEditorListener);
		editPartListener = new EditPartListener.Stub() {

			@Override
			public void selectedStateChanged(final EditPart editpart) {
				if (editpart.getSelected() != 2) {
					bringDown();
				}
			}
		};
		source.addEditPartListener(editPartListener);
	}

	/**
	 * Initializes the cell editor. This could be extended to set the initial
	 * text and add things such as {@link VerifyListener VerifyListeners}, if
	 * needed.
	 */
	private void initCellEditor() {
		// HINT Siehe JavaDoc
		cellEditor.setValue(((NamedElement) source.getModel()).getName());
	}

	private void setCellEditor(final CellEditor editor) {
		cellEditor = editor;
		if (cellEditor == null) {
			return;
		}
		hookListeners();
	}

	/**
	 * Unhooks listeners. Called from {@link #bringDown()}.
	 */
	private void unhookListeners() {
		source.removeEditPartListener(editPartListener);
		editPartListener = null;
		if (cellEditor == null) {
			return;
		}
		cellEditor.removeListener(cellEditorListener);
		cellEditorListener = null;
	}

}
