package roneditor.model;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

import ronmodel.HLArc;
import ronmodel.HLPlace;
import ronmodel.HLPlaceType;
import ronmodel.HLTransition;
import ronmodel.HLTransitionKind;
import ronmodel.ONNode;
import ronmodel.ONPlace;
import ronmodel.ONTransition;
import ronmodel.ObjectNet;
import ronmodel.Token;

/**
 * Provides some model specific tools for {@link HLTransition} such as checks
 * for validity and if the {@link HLTransition} can fire.
 */
public class HLTransitionUtil {

	/**
	 * Check if the {@link HLTransition} can fire.
	 * 
	 * @param transition
	 *            The transition to check
	 * @return true if the {@link HLTransition} can fire.
	 */
	public static boolean canFire(HLTransition transition) {
		switch (transition.getKind().getValue()) {
		case HLTransitionKind.APPLYRULE:
			return canFireApplyRule(transition);
		case HLTransitionKind.FIRE:
			return canFireFire(transition);
		case HLTransitionKind.SPLIT:
			return canFireSplit(transition);
		case HLTransitionKind.STANDARD:
			return canFireStandard(transition);
		}
		return false;
	}

	/**
	 * Check if the transition is valid, hence if its pre- and postconditions
	 * hold (this is type dependent).
	 * 
	 * @param transition
	 *            The {@link HLTransition} to check
	 * @return true if the transition is valid.
	 */
	public static boolean isValid(HLTransition transition) {
		switch (transition.getKind().getValue()) {
		case HLTransitionKind.APPLYRULE:
			return isValidApplyRule(transition);
		case HLTransitionKind.FIRE:
			return isValidFire(transition);
		case HLTransitionKind.SPLIT:
			return isValidSplit(transition);
		case HLTransitionKind.STANDARD:
			return isValidStandard(transition);
		}
		return false;
	}

	/**
	 * Return a {@link List} of all incoming {@link HLPlace} of the given
	 * {@link HLTransition} with {@link HLTransitionKind} <i>NET</i>.
	 * 
	 * @param transition
	 * @param placeType
	 * @return
	 */
	public static List<HLPlace> getIncomingNetPlaces(HLTransition transition) {
		return getIncomingPlaces(transition, HLPlaceType.NET);
	}

	/**
	 * Return a {@link List} of all incoming {@link HLPlace} of the given
	 * {@link HLTransition} with {@link HLTransitionKind} <i>RULE</i>.
	 * 
	 * @param transition
	 * @param placeType
	 * @return
	 */
	public static List<HLPlace> getIncomingRulePlaces(HLTransition transition) {
		return getIncomingPlaces(transition, HLPlaceType.RULE);
	}

	/**
	 * Return a {@link List} of all outgoing {@link HLPlace} of the given
	 * {@link HLTransition} with {@link HLTransitionKind} <i>NET</i>.
	 * 
	 * @param transition
	 * @param placeType
	 * @return
	 */
	public static List<HLPlace> getOutgoingNetPlaces(HLTransition transition) {
		return getOutgoingPlaces(transition, HLPlaceType.NET);
	}

	/**
	 * Return a {@link List} of all outgoing {@link HLPlace} of the given
	 * {@link HLTransition} with {@link HLTransitionKind} <i>RULE</i>.
	 * 
	 * @param transition
	 * @param placeType
	 * @return
	 */
	public static List<HLPlace> getOutgoingRulePlaces(HLTransition transition) {
		return getOutgoingPlaces(transition, HLPlaceType.RULE);
	}

	/**
	 * Collect all {@link Token} from the incoming <i>NET</i> places, also
	 * consider multiple {@link Token}s from the same place.
	 * 
	 * @param transition
	 *            The {@link HLTransition} to check
	 * @return
	 */
	public static List<ObjectNet> collectSourceNetTokens(HLTransition transition) {
		List<ObjectNet> tokens = new LinkedList<ObjectNet>();
		List<HLPlace> incomingNetPlaces = getIncomingNetPlaces(transition);
		HashMap<HLPlace, Integer> tokenizer = new HashMap<HLPlace, Integer>();
		for (HLPlace p : incomingNetPlaces) {
			List<ObjectNet> nets = p.getTokens();
			Integer neu = null;
			Integer counter = tokenizer.get(p);
			if (counter == null)
				neu = new Integer(0);
			else
				neu = new Integer(counter.intValue() + 1);
			tokenizer.put(p, neu);
			tokens.add(nets.get(neu.intValue()));
		}
		return tokens;
	}

	/**
	 * Return a {@link List} of all incoming {@link HLPlace} of the given
	 * {@link HLTransition} with the given {@link HLTransitionKind}.
	 * 
	 * @param transition
	 * @param placeType
	 * @return
	 */
	protected static List<HLPlace> getIncomingPlaces(HLTransition transition, int placeType) {
		List<HLArc> incomingArc = transition.getIncoming();
		List<HLPlace> incomingPlaces = new LinkedList<HLPlace>();
		for (HLArc arc : incomingArc) {
			HLPlace place = (HLPlace) arc.getSource();
			if (place.getType().getValue() == placeType)
				incomingPlaces.add(place);
		}
		return incomingPlaces;
	}

	/**
	 * Return a {@link List} of all outgoing {@link HLPlace} of the given
	 * {@link HLTransition} with the given {@link HLTransitionKind}.
	 * 
	 * @param transition
	 * @param placeType
	 * @return
	 */
	protected static List<HLPlace> getOutgoingPlaces(HLTransition transition, int placeType) {
		List<HLArc> outgoingArc = transition.getOutgoing();
		List<HLPlace> outgoingPlaces = new LinkedList<HLPlace>();
		for (HLArc arc : outgoingArc) {
			HLPlace place = (HLPlace) arc.getTarget();
			if (place.getType().getValue() == placeType)
				outgoingPlaces.add(place);
		}
		return outgoingPlaces;
	}

	/**
	 * Check if the transition of type <i>APPLYRULE</i> can fire
	 * 
	 * @param transition
	 *            The transition to check
	 * @return true if there is at least one token on every incoming place.
	 */
	protected static boolean canFireApplyRule(HLTransition transition) {
		// TODO: Check if the rule is applicable
		return canFireStandard(transition);
	}

	/**
	 * Check if the transition of type fire can fire
	 * 
	 * @param transition
	 *            The {@link HLTransition} to check
	 * @return true if there is at least one token on every incoming place and
	 *         the {@link ONTransition} can actually fire.
	 */
	protected static boolean canFireFire(HLTransition transition) {
		if (!canFireStandard(transition))
			return false;

		if (transition.getIncoming().size() != 1)
			return false;
		if (transition.getOutgoing().size() < 1)
			return false;
		HLArc a1 = (HLArc) transition.getIncoming().get(0);
		HLPlace p = (HLPlace) a1.getSource();

		for (Object net : p.getTokens()) {
			for (Object node : ((ObjectNet) net).getNodes()) {
				ONNode on = (ONNode) node;
				if (on instanceof ONPlace)
					continue;
				if (ONTransitionUtil.transitionCanFire((ONTransition) on)) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * Check if the transition of type split can fire.
	 * 
	 * @return true if there is at least one token on every incoming place.
	 */
	protected static boolean canFireSplit(HLTransition transition) {
		return canFireStandard(transition);
	}

	/**
	 * Check if the transition of type standard can fire.
	 * 
	 * @param transition
	 *            The {@link HLTransition} to check
	 * @return true if there is at least one token on every incoming place.
	 */
	protected static boolean canFireStandard(HLTransition transition) {
		List<HLArc> incomingArc = transition.getIncoming();
		for (HLArc arc : incomingArc) {
			HLPlace place = (HLPlace) arc.getSource();
			// no token on place? -> return
			if (place == null)
				return false;
			if (place.getTokens().size() == 0)
				return false;
		}
		return true;
	}

	/**
	 * Check if the {@link HLTransition} of the type <i>STANDARD</i> is valid .
	 * 
	 * @param transition
	 *            The {@link HLTransition} to check
	 * @return true if the {@link HLTransition} is valid
	 */
	protected static boolean isValidStandard(HLTransition transition) {
		// Collect incoming NET/RULE HLPlaces
		List<HLArc> incomingArc = transition.getIncoming();
		int incomingNetPlaces = 0;
		int incomingRulePlaces = 0;
		for (HLArc arc : incomingArc) {
			HLPlace place = (HLPlace) arc.getSource();
			// this should not happen
			if (place == null)
				return false;
			if (place.getType().getValue() == HLPlaceType.NET)
				incomingNetPlaces++;
			else
				incomingRulePlaces++;
		}

		// Do not execute with more than on RULE
		if (incomingRulePlaces > 1)
			return false;

		// Collect outgoing NET/RULE HLPlaces
		List<HLArc> outgoingArc = transition.getOutgoing();
		int outgoingNetPlaces = 0;
		int outgoingRulePlaces = 0;
		for (HLArc arc : outgoingArc) {
			HLPlace place = (HLPlace) arc.getTarget();
			if (place == null)
				return false;
			if (place.getType().getValue() == HLPlaceType.NET)
				outgoingNetPlaces++;
			else
				outgoingRulePlaces++;
		}

		if (incomingRulePlaces == 0 && outgoingRulePlaces > 0)
			return false;
		if (incomingNetPlaces == 0 && outgoingNetPlaces > 0)
			return false;

		return true;
	}

	/**
	 * Check if the {@link HLTransition} of the type <i>APPLYRULE</i> is valid .
	 * 
	 * @param transition
	 *            The {@link HLTransition} to check
	 * @return true if the {@link HLTransition} is valid
	 */
	protected static boolean isValidApplyRule(HLTransition transition) {
		// Collect incoming NET/RULE HLPlaces
		List<HLArc> incomingArc = transition.getIncoming();
		Integer incomingRulePlaces = 0;
		Integer incomingNetPlaces = 0;
		for (HLArc arc : incomingArc) {
			HLPlace place = (HLPlace) arc.getSource();
			// this should not happen..
			if (place == null)
				return false;
			// more than one RULE Place? -> return
			if (incomingRulePlaces > 1)
				return false;
			if (place.getType().getValue() == HLPlaceType.NET)
				incomingNetPlaces++;
			else
				incomingRulePlaces++;
		}

		// Do not execute with less than 1 (one) OBJECTNET
		if (incomingNetPlaces == 0)
			return false;

		// Do not execute with more or less than 1 (one) RULE
		if (incomingRulePlaces != 1)
			return false;

		return true;
	}

	/**
	 * Check if the {@link HLTransition} of the type <i>FIRE</i> is valid .
	 * 
	 * @param transition
	 *            The {@link HLTransition} to check
	 * @return true if the {@link HLTransition} is valid
	 */
	protected static boolean isValidFire(HLTransition transition) {
		// test if only one NET place is in pre and only NETs are post scope
		if (transition.getIncoming().size() != 1)
			return false;
		if (transition.getOutgoing().size() < 1)
			return false;
		HLArc a1 = (HLArc) transition.getIncoming().get(0);
		HLPlace p = (HLPlace) a1.getSource();
		if (p == null)
			return false;
		if (p.getType() == HLPlaceType.RULE_LITERAL)
			return false;
		for (Object t : transition.getOutgoing()) {
			HLPlace pl = (HLPlace) ((HLArc) t).getTarget();
			if (pl == null)
				return false;
			if (pl.getType() == HLPlaceType.RULE_LITERAL)
				return false;
		}

		return true;
	}

	/**
	 * Check if the {@link HLTransition} of the type <i>SPLIT</i> is valid .
	 * 
	 * @param transition
	 *            The {@link HLTransition} to check
	 * @return true if the {@link HLTransition} is valid
	 */
	protected static boolean isValidSplit(HLTransition transition) {
		// TODO: Streamline this! (Eliminate redundancy to canFire etc.)
		if (transition.getIncoming().size() != 1)
			return false;
		if (transition.getOutgoing().size() != 1)
			return false;

		// Get connected HLPlaces
		HLPlace sourcePlace = (HLPlace) ((HLArc) (transition.getIncoming().get(0))).getSource();
		HLPlace targetPlace = (HLPlace) ((HLArc) (transition.getOutgoing().get(0))).getTarget();

		// Only continue if we have NET HLPlaces
		if (sourcePlace == null || sourcePlace.getType().getValue() != HLPlaceType.NET)
			return false;
		if (targetPlace == null || targetPlace.getType().getValue() != HLPlaceType.NET)
			return false;

		return true;
	}

}
