package roneditor.model;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.Assert;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.impl.AdapterImpl;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PlatformUI;

public class IDUtil {

	public static IWorkbenchPage page; 
	/**
	 * A map to resolve unique ids to ecore models. This is used to allow views
	 * opened to show a specific model to resolve their secondary id to the
	 * model to be shown.
	 */
	public static final Map<String, EObject> id2ModelMap = new HashMap<String, EObject>();

	/**
	 * A map reversing id2ModelMap to get possible previously generated ids for
	 * models rather that create new ones.
	 */
	public static final Map<EObject, String> model2IdMap = new HashMap<EObject, String>();

	/**
	 * A map only used to be able to remove deletion listeners from the models
	 * when the editor is being closed.
	 */
	public static final Map<EObject, Adapter> model2ListenerMap = new HashMap<EObject, Adapter>();

	/**
	 * Generates an unique ID for this model and registers them with each other
	 * in two maps. The ID is to be used as secondary ID to tell views what
	 * model they shall display. If the model has already been registered
	 * before, the previous generated ID is returned rather than a newly created
	 * one. <br>
	 * 
	 * Additionally an {@link Adapter} is added to the model. The adapter will
	 * close the views that have been opened with this ID as secondary ID
	 * automatically when deletion of the model is being notified.
	 * 
	 * @param model
	 *            The model to get an ID for
	 * @return A unique ID for the model
	 */
	public static String createIDForModel(final EObject model) {
		return createIDForModel(model, null);
	}

	/**
	 * Generates an unique ID for this model and registers them with each other
	 * in two maps. The ID is to be used as secondary ID to tell views what
	 * model they shall display. If the model has already been registered
	 * before, the previous generated ID is returned rather than a newly created
	 * one. <br>
	 * 
	 * Additionally an {@link Adapter} is added to the model. The adapter will
	 * close the views that have been opened with this ID as secondary ID
	 * automatically when deletion of the model is being notified. *
	 * 
	 * @param model
	 *            The model to get an ID for
	 * @param forceID
	 *            May be used to force a particular ID for a model. If
	 *            <code>null</code>, a new ID will be generated.
	 * @return A unique ID for the model
	 */
	public static String createIDForModel(final EObject model, String forceID) {
		if (model2IdMap.containsKey(model)) {
			return model2IdMap.get(model);
		}

		// listen to model for deletion
		final Adapter listener = new AdapterImpl() {
			/*
			 * (non-Javadoc)
			 * 
			 * @see org.eclipse.emf.common.notify.impl.AdapterImpl#notifyChanged(org.eclipse.emf.common.notify.Notification)
			 */
			@Override
			public void notifyChanged(Notification msg) {
				// Test if model has been deleted from the resource
				// Maybe the notification could tell somehow?
				if (null == model.eResource()) {
					// close views showing deleted models
					closeViewsShowing(model);
				}
			}
		};

		// add and register adapter
		model.eAdapters().add(listener);
		model2ListenerMap.put(model, listener);

		// generate new ID if not forced
		if (null == forceID) {
			forceID = EcoreUtil.generateUUID();
		}

		// check for uniqueness of ID, should never occur
		Assert.isTrue(!id2ModelMap.containsKey(forceID),
				"Generated or forced ID has already been assigned to a model!");

		// register crosswise
		model2IdMap.put(model, forceID);
		id2ModelMap.put(forceID, model);
		return forceID;
	}
	
	/**
	 * Closes alls views in the site's page whose secondary ID correspond to the
	 * id the passed model has been registered with.
	 * 
	 * @param model
	 *            The model
	 * @return The (primary) id of the last closed view showing the model if
	 *         any, else <code>null</code>
	 */
	public static String closeViewsShowing(final EObject model) {
		//final IWorkbenchPage page = getSite().getPage();
		final String modelID = model2IdMap.get(model);
		
		String foundViewID = null;
		if (page!=null){
			final IViewReference[] viewRefs = page.getViewReferences();
			for (final IViewReference viewRef : viewRefs) {
				if (modelID.equals(viewRef.getSecondaryId())) {
					page.hideView(viewRef.getView(true));
					foundViewID = viewRef.getId();
				}
			}
		}
		return foundViewID;
	}

}
