package roneditor.model;

import java.util.Collection;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Vector;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.util.EcoreUtil;

import ronmodel.ONArc;
import ronmodel.ONNode;
import ronmodel.ObjectNet;
import ronmodel.Token;

public class ObjectNetUtil {

	/**
	 * Traverse the tree recursively and tag all unvisited nodes in the given
	 * {@link Collection}.
	 * 
	 * @param list
	 *            {@link Collection} of tagged {@link ONNode}s
	 * @param node
	 *            {@link ONNode} to traverse.
	 */
	private static void visitNode(Collection<ONNode> list, ONNode node) {
		list.add(node);
		for (Object arcObj : node.getOutgoing()) {
			ONArc arc = (ONArc) arcObj;
			if (!list.contains(arc.getTarget())) {
				visitNode(list, arc.getTarget());
			}
		}
		for (Object arcObj : node.getIncoming()) {
			ONArc arc = (ONArc) arcObj;
			if (!list.contains(arc.getSource())) {
				visitNode(list, arc.getSource());
			}
		}
	}

	/**
	 * Take the first node from an {@link ObjectNet} and build it's spanning
	 * tree, then return the connected component derived from the spanning tree
	 * to a new {@link ObjectNet} and return it.
	 * 
	 * @param on
	 *            {@link ObjectNet} to split off the first connected component
	 * @return The new {@link ObjectNet} with the first connected component
	 */
	private static ObjectNet splitFirst(ObjectNet on) {
		ObjectNet result = RONModelManager.getFactory().createObjectNet();
		EList nodes = on.getNodes();
		ONNode n0 = (ONNode) nodes.get(0);
		HashSet<ONNode> list = new HashSet<ONNode>();
		visitNode(list, n0);
		moveConnectedComponent(list, result);
		result.setName(on.getName());
		return result;
	}

	/**
	 * Performs an very lazy split operation on an {@link ObjectNet}, hence
	 * splits all connected components into a {@link List} of {@link ObjectNet}.
	 * 
	 * @param on
	 * @return
	 */
	public static Collection<ObjectNet> split(ObjectNet on) {
		LinkedList<ObjectNet> result = new LinkedList<ObjectNet>();
		if (on.getNodes() == null)
			return result;
		while (on.getNodes().size() > 0) {
			result.add(splitFirst(on));
		}
		return result;
	}

	/**
	 * Move an connected component to the target {@link ObjectNet}, hence move
	 * all the {@link ONNode} in the given {@link Collection} with it's outgoing
	 * {@link ONArc} to the target {@link ObjectNet}.
	 * 
	 * @param nodes
	 * @param target
	 */
	@SuppressWarnings("unchecked")
	public static void moveConnectedComponent(Collection<ONNode> nodes, ObjectNet target) {
		// Copy as we change the list while iterating
		Vector<ONNode> nodesCopy = new Vector<ONNode>(nodes);
		for (ONNode node : nodesCopy) {
			node.setObjectNet(target);
			Vector<ONArc> outgoing = new Vector<ONArc>(node.getOutgoing());
			for (ONArc arc : outgoing) {
				arc.setObjectNet(target);
			}
		}

	}


	public static void unionIntoObjectNet(List<ObjectNet> tokens, ObjectNet target) {
		String targetName = "";
		Collection allTokenContent = null;
		for (Token net : tokens) {
			if (allTokenContent == null) {
				allTokenContent = EcoreUtil.copyAll(net.eContents());
				targetName = net.getName();
			} else
				allTokenContent.addAll(EcoreUtil.copyAll(net.eContents()));
		}
		target.setName(targetName);
		target.setPlace(null);
		for (Object object : allTokenContent) {
			if (object instanceof ONNode) {
				((ONNode) object).setObjectNet(target);
			} else if (object instanceof ONArc) {
				((ONArc) object).setObjectNet(target);
			}
		}
	}


}
