package roneditor.model;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.ui.IEditorPart;

import ronmodel.LHS;
import ronmodel.Mapping;
import ronmodel.NAC;
import ronmodel.NamedElement;
import ronmodel.ONArc;
import ronmodel.ONNode;
import ronmodel.ONPlace;
import ronmodel.ONTransition;
import ronmodel.ObjectNet;
import ronmodel.RHS;
import ronmodel.Rule;

public class RuleChecker {

	public static IEditorPart editor;
	static Rule rule;
	
	
	/**
	 * Delete all existing <code>IMarker.PROBLEM</code> markers.
	 */
	public static void deleteAllMarkers() {
		IResource resource = (IResource) editor.getAdapter(IResource.class);
		if (resource == null)
			return;

		try {
			resource.deleteMarkers(IMarker.PROBLEM, false, 1);
		} catch (CoreException e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * Create an eclipse error marker on given location with specified message.
	 * 
	 * @param location
	 * @param message
	 */
	public static void createErrorMarker(String location, String message) {
		IResource resource = (IResource) editor.getAdapter(IResource.class);
		if (resource == null)
			return;
		try {
			IMarker marker = resource.createMarker(IMarker.PROBLEM);
			marker.setAttribute(IMarker.MESSAGE, location + ": " + message);
			marker.setAttribute(IMarker.SEVERITY, IMarker.SEVERITY_ERROR);
			marker.setAttribute(IMarker.LOCATION, location);

		} catch (CoreException e) {
			throw new RuntimeException(e);
		}
	}
	
	protected static List<Mapping> getAllMappings() {
		return RuleUtil.getAllMappings(rule);
	}
	
	
	/**
	 * Check if the mappings are compatible, hence if the source and target is
	 * of the same type
	 */
	protected static void checkCompatibility() {
		List<Mapping> mappings = getAllMappings();
		for (Mapping mapping : mappings) {
			if (mapping.getSource().getClass() != mapping.getTarget().getClass()) {
				createErrorMarker(mapping.getSource().getName() + "->" + mapping.getTarget().getName(),
						"Mapping is not compatible (" + mapping.getSource().getClass() + "->" + mapping.getTarget().getClass()
								+ ")");
			}
			//seb: are source and target name the same? 
			//( if sourcename not empty folie 09RuleEditor.pdf page 7 )
			if (mapping.getTarget().getName().compareTo("")!=0 &&
					mapping.getTarget().getName().compareTo(mapping.getSource().getName())!=0 ) {
				createErrorMarker(mapping.getSource().getName() + "->" + mapping.getTarget().getName(),
						"Names are not compatible");				
			}
			//seb: are source and target capacity the same? 
			if (mapping.getTarget() instanceof ONPlace &&
					((ONPlace)mapping.getTarget()).getCapacity() !=
						((ONPlace)mapping.getSource()).getCapacity() ) {
				createErrorMarker(mapping.getSource().getName() + "->" + mapping.getTarget().getName(),
						"Capacities are not compatible");				
			}
		}
	}
	
	/**
	 * Helper function for checking injectivity, performs actual checks on the
	 * mapping for one-to-one relation.
	 * 
	 * @param list of mappings
	 */
	protected static void checkMappingsForInjectivity(List list) {
		HashMap<ONNode, Mapping> targets = new HashMap<ONNode, Mapping>();
		HashMap<ONNode, Mapping> sources = new HashMap<ONNode, Mapping>();
		for (Object mappingObject : list) {
			Mapping mapping = (Mapping) mappingObject;
			ONNode source = mapping.getSource();
			ONNode target = mapping.getTarget();
			Mapping sourceMapping = sources.get(target);
			if (sourceMapping != null) {
				createErrorMarker(source.getName() + "->_" + target.getName() + "_", "There is already a mapping "
						+ sourceMapping.getSource().getName() + "->" + sourceMapping.getTarget().getName());
			}
			Mapping targetMapping = targets.get(target);
			if (targetMapping != null) {
				createErrorMarker("_" + source.getName() + "_->" + target.getName(), "There is already a mapping "
						+ targetMapping.getSource().getName() + "->" + targetMapping.getTarget().getName());
			}
			sources.put(source, mapping);
			targets.put(target, mapping);
		}
	}
	
	/**
	 * Check the mappings for injectivity hence if the one-to-one relation of
	 * the mapping holds. If there are inconsistencies, an Eclipse
	 * {@link IMarker} is generated.
	 */
	protected static void checkMappingsForInjectivity() {
		checkMappingsForInjectivity(rule.getRhs().getMappings());
		for (Object nacObject : rule.getNacs()) {
			NAC nac = (NAC) nacObject;
			checkMappingsForInjectivity(nac.getMappings());
		}
	}
	/**
	 * Check if pre and post conditions hold. Formal: f_P^{\oplus} \circ pre_1 =
	 * pre_2 \circ f_t f_P^{\oplus} \circ post_1 = post_2 \circ f_t
	 */
	@SuppressWarnings("unchecked")
	protected static void checkPreAndPostConditions() {
		List <Mapping> mappings = new LinkedList<Mapping>();
		for ( Object mappingObject : rule.getRhs().getMappings() ) {
			mappings.add((Mapping)mappingObject);
		}
		checkPreAndPostConditions(mappings);
		for ( Object nacObject : rule.getNacs() ) {
			NAC nac = (NAC)nacObject;
			mappings.clear();
			for ( Object mappingObject : nac.getMappings() ) {
				mappings.add((Mapping)mappingObject);
			}
			checkPreAndPostConditions(mappings);
		}
	}
	
	protected static String getNodeLocation(ONNode node) {
		ObjectNet network = node.getObjectNet();
		String nodeLocation = node.getName()+"@"+network.getName(); 
		NamedElement container = (NamedElement)network.eContainer();
		if ( container != null ) {
			nodeLocation += " ["+container.getName()+"]";
		}
		else {
			nodeLocation += " [unknown]";
		}
		return nodeLocation;
		
	}

	
	@SuppressWarnings("unchecked")
	protected static void checkPreAndPostConditions(List<Mapping> mappings) {
		//prepare helper lists and maps
		HashMap<ONNode, ONNode> splaces = new HashMap<ONNode, ONNode>();
		HashMap<ONNode, ONNode> strans = new HashMap<ONNode, ONNode>();
		List<ONNode> transitions = new LinkedList<ONNode>();
		for (Mapping m : mappings) {
			ONNode node = m.getSource();
			if (node instanceof ONTransition) {
				strans.put(node, m.getTarget());
				transitions.add(node);
			}
			else splaces.put(node, m.getTarget());
		}
		
		//check pre-conditions
		for (ONNode n : transitions) {
			ONNode t = strans.get(n);
			//for pre-check
			List<ONArc> sarcs = n.getIncoming();
			List<ONArc> tarcs = t.getIncoming();
			HashMap<ONNode,ONArc> right = new HashMap<ONNode,ONArc>();
			for (ONArc i : tarcs)
				right.put( i.getSource(), i );
			for (ONArc i : sarcs){
				ONNode node = splaces.get(i.getSource());
				if (node == null) continue; 
				ONArc arc = right.get(node);
				if (arc == null ) { createErrorMarker(getNodeLocation(n), "No corresponding Arc from Place " + i.getSource().getName()+ " found in Pre-Condition ("+	node.getObjectNet().getName() + ")"); continue; }
				if (arc.getWeight() != i.getWeight()) 
					createErrorMarker(getNodeLocation(n), "Different weights for Arcs from Place "+i.getSource().getName() + " and Place "+arc.getSource().getName()+ " in Post-Condition ("
							+	node.getObjectNet().getName() + ")");
				right.remove(node);
			}
			if (!right.isEmpty()) createErrorMarker(getNodeLocation(n), "Missing Arc(s) in Pre-Conditions (lhs)");
			
			//for post-check
			List<ONArc> psarcs = n.getOutgoing();
			List<ONArc> ptarcs = t.getOutgoing();
			HashMap<ONNode,ONArc> pright = new HashMap<ONNode,ONArc>();
			for (ONArc i : ptarcs)
				pright.put( i.getTarget(), i );
			for (ONArc i : psarcs){
				ONNode node = splaces.get(i.getTarget());
				if (node == null) continue; 
				ONArc arc = pright.get(node);
				if (arc == null ) { createErrorMarker(getNodeLocation(n), "No corresponding Arc to Place " + i.getTarget().getName()+ " found in Post-Condition ("+	node.getObjectNet().getName() + ")"); continue; }
				if (arc.getWeight() != i.getWeight()) 
					createErrorMarker(getNodeLocation(n), "Different weights for Arcs to Place "+i.getTarget().getName() + " and Place "+arc.getTarget().getName() + " in Post-Condition ("
							+	node.getObjectNet().getName() + ")");
				pright.remove(node);
			}
			if (!pright.isEmpty()) createErrorMarker(getNodeLocation(n), "Missing Arc(s) in Post-Condition (lhs)");
		}
	}
	
	/**
	 * Check if a Name of an ObejectNet Element is used more than once.
	 */
	protected static void checkNames(){
		checkNames( ((RHS)rule.getRhs()).getNodes() );
		checkNames( ((LHS)rule.getLhs()).getNodes() );
		for (Object nac : rule.getNacs())
			checkNames( ((NAC)nac).getNodes() );
			
	}

	protected static void checkNames(List l){
		List<String> names = new LinkedList<String>();
		for (Object n : l){
			ONNode node = (ONNode)n;
			if (names.contains(node.getName())) createErrorMarker(getNodeLocation(node), "Name already in use.");
				if (node.getName().compareTo("")!=0) names.add(node.getName());
		}
	}
	
	public static boolean isRuleOk(IEditorPart e, Rule r) {
		editor = e;
		rule = r;
		
		deleteAllMarkers();
		checkCompatibility();
		checkMappingsForInjectivity();
		checkPreAndPostConditions();
		//checkNames();
		IResource resource = (IResource) editor.getAdapter(IResource.class);
		
		//seb: simple hacky solution
		IMarker[] markers = null;
		try {
			markers = resource.findMarkers(IMarker.PROBLEM, false, 1);
		} catch (CoreException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		return (markers != null) && (markers.length==0);
	}
}
