package roneditor.model;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;

import ronmodel.Codomain;
import ronmodel.LHS;
import ronmodel.Mapping;
import ronmodel.NAC;
import ronmodel.ONNode;
import ronmodel.ObjectNet;
import ronmodel.RHS;
import ronmodel.Rule;

/**
 * Helper Class for {@link Rule} specific operations. Partly provides functionality 
 * for the view in form of a color association of mappings. These mappings are
 * stored in a {@link Map}, and have to be updated by a appropriate controller
 * on model changes.
 */
public class RuleUtil {

	/**
	 * Provides a {@link Map} from {@link Rule}s to another {@link Map} that
	 * stores the {@link Color}s for for each mapped {@link ONNode} in that
	 * {@link Rule}
	 */
	private static Map<Rule, Map<ONNode, Color>> ruleNodeColorMap = new HashMap<Rule, Map<ONNode, Color>>();

	/**
	 * Simple {@link Set} to mark the {@link Rule}s whose color map is
	 * up-to-date. {@link Rule}s <i>not</i> in the map will be updated on the
	 * next call of {@link RuleUtil#getColorForNode(Rule, ONNode)}
	 */
	private static Set<Rule> ruleColorMapClean = new HashSet<Rule>();

	/**
	 * Build the {@link Color} map for mappings on a {@link Rule}. <br/> This
	 * function will generate a list of distinctive {@link Color}s for each
	 * mapping on a given {@link Rule}. It will check all mappings (for all
	 * active {@link Codomain}s) and tries to reuse {@link Color}s where
	 * possible.
	 * 
	 * @param rule  
	 */
	private static void buildColorMap(Rule rule) {
		long t = System.currentTimeMillis();
		//System.out.println("RuleUtil.buildColorMap() ==> ");
		Set<ONNode> leftNodes = new HashSet<ONNode>();
		Map<ONNode, Color> oldColorMap = ruleNodeColorMap.get(rule);
		Map<ONNode, Color> newColorMap = new HashMap<ONNode, Color>();
		List<Mapping>  mappings = getAllMappings(rule); 
		for (Mapping m : mappings) {
			leftNodes.add(m.getSource());
		}
		// spread color along hue spectrum evenly..
		if (oldColorMap != null) {
			for (Object nodeObject : oldColorMap.keySet()) {
				if (leftNodes.contains(nodeObject)) {
					newColorMap.put((ONNode) nodeObject, (Color) oldColorMap.get(nodeObject));
					leftNodes.remove(nodeObject);
				}
			}
		}
		Collection<Color> colorCollection = newColorMap.values();
		int c = 1;
		for (ONNode leftNode : leftNodes) {
			Color color;
			do {
				float f = (float) Math.ceil((float) (c) / 360.0f); // step in
				// value
				// direction
				color = new Color(null, new RGB(c % 360, 0.7f / f, 1.0f));
				/*
				 * we may try one of these nice prime-numbers too 2, 3, 5, 7,
				 * 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 53, 59, 61, 67,
				 * 71, 73, 79, 83, 89, 97, 101, 103, 107, 109, 113
				 */
				c += 47; // step in hue direction
			} while (colorCollection.contains(color));
			newColorMap.put(leftNode, color);
		}
		for (Mapping m : mappings) {
			newColorMap.put(m.getTarget(), newColorMap.get(m.getSource()));
		}
		ruleNodeColorMap.remove(rule);
		ruleNodeColorMap.put(rule, newColorMap);
		ruleColorMapClean.add(rule);
		//System.out.println("RuleUtil.buildColorMap() <== "+(t-System.currentTimeMillis()));
	}

	/**
	 * Mark the color map for the given {@link Rule} as dirty, hence it will be
	 * regenerated on the next call to
	 * {@link RuleUtil#getColorForNode(Rule, ONNode)} with the same rule.
	 * 
	 * @param rule
	 */
	public static void markColorMapDirty(Rule rule) {
		ruleColorMapClean.remove(rule);
	}

	/**
	 * Get the {@link Rule} for a {@link ONNode}.
	 * 
	 * @param node
	 * @return The {@link Rule} associated to a {@link ONNode},
	 *         <code>null</code> if the {@link ONNode} is not in a
	 *         {@link Rule}.
	 */
	public static Rule getRuleFromONNode(ONNode node) {
		ObjectNet on = node.getObjectNet();
		Rule rule = null;

		if (on instanceof LHS) {
			rule = ((LHS) on).getRule();
		} else if (on instanceof RHS) {
			rule = ((RHS) on).getRule();
		} else if (on instanceof NAC) {
			rule = ((NAC) on).getRule();
		}
		return rule;
	}

	/**
	 * Return the {@link Color} for a mapping of two nodes in a given
	 * {@link Rule}, generated by {@link RuleUtil#buildColorMap(Rule)}.
	 *
	 * @param rule The {@link Rule} that contains the mappings
	 * @param node The {@link ONNode} to get the color for
	 * @return color for the node
	 */
	public static Color getColorForNode(Rule rule, ONNode node) {
		//System.out.println("RuleUtil.getColorForNode("+rule+","+node+")");
		if (!ruleColorMapClean.contains(rule)) {
			buildColorMap(rule);
		}
		Map<ONNode, Color> colorMap = ruleNodeColorMap.get(rule);
		Color c = null;
		if (colorMap == null || (c = colorMap.get(node)) == null) {
			c = new Color(null, 255, 255, 255);
		}
		return c;
	}

	/**
	 * Return all mappings for a rule
	 * 
	 * @param rule
	 * @return List of all mappings that are in the rule
	 */
	public static List<Mapping> getAllMappings(Rule rule) {
		List<Mapping> mappings = new LinkedList<Mapping>();
		for (Object mappingObject : rule.getRhs().getMappings()) {
			mappings.add((Mapping) mappingObject);
		}
		for (Object nacObject : rule.getNacs()) {
			NAC nac = (NAC) nacObject;
			for (Object mappingObject : nac.getMappings()) {
				mappings.add((Mapping) mappingObject);
			}
		}
		return mappings;
	}

}
