package roneditor.ui.part;

import org.eclipse.core.runtime.Assert;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPartFactory;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.KeyHandler;
import org.eclipse.gef.KeyStroke;
import org.eclipse.gef.palette.PaletteRoot;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.action.IAction;
import org.eclipse.swt.SWT;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IPropertyListener;
import org.eclipse.ui.actions.ActionFactory;

import roneditor.gef.actions.CopyModelToClipboardAction;
import roneditor.gef.actions.PasteModelAction;
import roneditor.gef.actions.objectnet.FireTransitionAction;
import roneditor.gef.actions.objectnet.MoveNodeAction;
import roneditor.gef.actions.objectnet.ObjectNetEditPartGraphLayoutAction;
import roneditor.gef.editparts.objectnet.ObjectNetEditPartFactory;
import roneditor.ui.contextmenu.ContextMenuProviderWithActionRegistry;
import roneditor.ui.contextmenu.ObjectNetPageContextMenuProvider;
import roneditor.ui.views.AbstractPageBookView;
import ronmodel.ONNode;
import ronmodel.ObjectNet;

/**
 * <p>
 * This is an implementation of {@link AbstractPageBookView} to display
 * {@link ObjectNet}s.
 * </p>
 * 
 * <p>
 * It uses {@link ObjectNetEditPartFactory} and
 * {@link ObjectNetPageContextMenuProvider}, configures the keyhandler and
 * creates actions to move {@link ONNode}s by pressing keys.
 * </p>
 * 
 * <p>
 * On model change nothing is done.
 * </p>
 * 
 * @author Tony Modica
 */
public class ObjectNetPage extends OwnAbstractMultiViewerPage implements IPropertyListener {

	public ObjectNetPage(final ObjectNet objectNet, final IEditorPart editor) {
		super(objectNet, editor);
		// getSite().getWorkbenchWindow().getWorkbench().addPropertyListener(this);
		// System.err.println("created ObjectNetPage for " + objectNet);
	}

	/**
	 * Register an Action from the Editor in the local ActionRegistry and set
	 * the global action handler for the site for this very action
	 * 
	 * @param id
	 *            the identifier for the editor registry
	 */
	protected void registerEditorAction(final String id) {
		ActionRegistry editorRegistry = (ActionRegistry) getEditor().getAdapter(ActionRegistry.class);
		Assert.isNotNull(editorRegistry, "The editor did not deliver a ActionRegistry instance for the page!");

		IAction action = editorRegistry.getAction(id);
		Assert.isNotNull(action, "The editor did not deliver a ObjectNetEditPartGraphLayoutAction instance for the page!");
		getActionRegistry().registerAction(action);

		registerHandleForEditorAction(id);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#createActions()
	 */
	@Override
	protected void createActions() {
		// HINT Hier werden Instanzen der Actions erzeugt und registriert,
		// damit der ContextMenuProvider sie in das Kontextmen� einf�gen kann.

		getEditor().addPropertyListener(this);

		SelectionAction selectionAction;
		selectionAction = new MoveNodeAction(getEditor(), MoveNodeAction.LEFT);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(), MoveNodeAction.RIGHT);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(), MoveNodeAction.UP);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(), MoveNodeAction.DOWN);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(), MoveNodeAction.PREC_LEFT);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(), MoveNodeAction.PREC_RIGHT);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(), MoveNodeAction.PREC_UP);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(), MoveNodeAction.PREC_DOWN);
		getActionRegistry().registerAction(selectionAction);

		// We want to have the graph layout and fire transition and delete
		// actions here
		registerEditorAction(ObjectNetEditPartGraphLayoutAction.ID);
		registerEditorAction(FireTransitionAction.ID);
		registerEditorAction(ActionFactory.DELETE.getId());
		
		//Paste ObjectNets
		selectionAction = new PasteModelAction(getEditor());
		getActionRegistry().registerAction(selectionAction);
		registerHandleForEditorAction(ActionFactory.PASTE.getId());
		
		//Copy ObjectNets
		selectionAction = new CopyModelToClipboardAction(getEditor());
		getActionRegistry().registerAction(selectionAction);
		registerHandleForEditorAction(ActionFactory.COPY.getId());
		
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#setupKeyHandler(org.eclipse.gef.KeyHandler)
	 */
	@Override
	protected void setupKeyHandler(final KeyHandler kh) {
		// Arrow keys run the correspondent MoveNodeActions
		kh.put(KeyStroke.getPressed(SWT.ARROW_LEFT, 0), (getActionRegistry().getAction(MoveNodeAction.LEFT)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_RIGHT, 0), (getActionRegistry().getAction(MoveNodeAction.RIGHT)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_UP, 0), (getActionRegistry().getAction(MoveNodeAction.UP)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_DOWN, 0), (getActionRegistry().getAction(MoveNodeAction.DOWN)));

		// SHIFT + Arrow keys run the correspondent precise MoveNodeActions
		kh.put(KeyStroke.getPressed(SWT.ARROW_LEFT, SWT.SHIFT), (getActionRegistry().getAction(MoveNodeAction.PREC_LEFT)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_RIGHT, SWT.SHIFT), (getActionRegistry().getAction(MoveNodeAction.PREC_RIGHT)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_UP, SWT.SHIFT), (getActionRegistry().getAction(MoveNodeAction.PREC_UP)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_DOWN, SWT.SHIFT), (getActionRegistry().getAction(MoveNodeAction.PREC_DOWN)));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#createEditPartFactory()
	 */
	@Override
	protected EditPartFactory createEditPartFactory() {
		return new ObjectNetEditPartFactory();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#getContextMenuProvider()
	 */
	@Override
	protected ContextMenuProviderWithActionRegistry createContextMenuProvider(final EditPartViewer viewer) {
		return new ObjectNetPageContextMenuProvider(viewer, getActionRegistry());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#createPaletteRoot()
	 */
	@Override
	protected PaletteRoot createPaletteRoot() {
		// ObjectNetEditorPalette
		return new roneditor.gef.palette.objectnet.ObjectNetPaletteRoot();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#getThumbSashWeights()
	 */
	@Override
	protected int[] getThumbSashWeights() {
		return new int[] { 3, 1 };
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.emf.common.notify.Adapter#notifyChanged(org.eclipse.emf.common.notify.Notification)
	 */
	public void notifyChanged(final Notification notification) {
		
		// System.out.println("ObjectNetPage.notifyChanged()"+notification);
		
		// the page doesn't care for model changes (yet)
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.ruleview.AbstractMultiViewerPage#getNumberOfViewers()
	 */
	@Override
	public int getNumberOfViewers() {
		return 1;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.ruleview.AbstractMultiViewerPage#getViewerContents()
	 */
	@Override
	public EObject[] getViewerContents() {
		return new EObject[] { getModel() };
	}

	public void propertyChanged(Object source, int propId) {
		System.out.println("ObjectNetPage.propertyChanged(" + source + ")");

		// TODO Auto-generated method stub

	}

}
