package roneditor.ui.part;

import org.eclipse.core.runtime.Assert;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPartFactory;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.KeyHandler;
import org.eclipse.gef.KeyStroke;
import org.eclipse.gef.palette.PaletteRoot;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.swt.SWT;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IPropertyListener;
import org.eclipse.ui.actions.ActionFactory;

import roneditor.gef.actions.CopyModelToClipboardAction;
import roneditor.gef.actions.PasteModelAction;
import roneditor.gef.actions.objectnet.MoveNodeAction;
import roneditor.gef.actions.ron.ApplyRuleDialogAction;
import roneditor.gef.actions.ron.FireDialogAction;
import roneditor.gef.editparts.ron.RONEditPartFactory;
import roneditor.gef.palette.ron.RONPaletteRoot;
import roneditor.ui.contextmenu.ContextMenuProviderWithActionRegistry;
import roneditor.ui.contextmenu.RONEditorContextMenuProvider;
import ronmodel.RON;

/**
 * @author Tony Modica
 */
public class RONPage extends OwnAbstractMultiViewerPage implements IPropertyListener {

	public RONPage(final RON ron, final IEditorPart editor) {
		super(ron, editor);
	}

	/**
	 * Register an Action from the Editor in the local ActionRegistry and set
	 * the global action handler for the site for this very action
	 * 
	 * @param id
	 *            the identifier for the editor registry
	 */
	protected void registerEditorAction(final String id) {
		ActionRegistry editorRegistry = (ActionRegistry) getEditor().getAdapter(ActionRegistry.class);
		Assert.isNotNull(editorRegistry, "The editor did not deliver a ActionRegistry instance for the page!");

		IAction action = editorRegistry.getAction(id);
		Assert.isNotNull(action, "The editor did not deliver a RONEditPartGraphLayoutAction instance for the page!");
		getActionRegistry().registerAction(action);

		registerHandleForEditorAction(id);
		
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#createActions()
	 */
	@Override
	protected void createActions() {
		getEditor().addPropertyListener(this);

		SelectionAction selectionAction;
		selectionAction = new MoveNodeAction(getEditor(), MoveNodeAction.LEFT);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(), MoveNodeAction.RIGHT);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(), MoveNodeAction.UP);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(), MoveNodeAction.DOWN);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(), MoveNodeAction.PREC_LEFT);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(), MoveNodeAction.PREC_RIGHT);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(), MoveNodeAction.PREC_UP);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(), MoveNodeAction.PREC_DOWN);
		getActionRegistry().registerAction(selectionAction);
		
		final Action copyObjectNetToClipboardAction = new CopyModelToClipboardAction(getEditor());
		getActionRegistry().registerAction(copyObjectNetToClipboardAction);
		registerHandleForEditorAction(ActionFactory.COPY.getId());
		
		final Action pasteObjectNetAction = new PasteModelAction(getEditor());
		getActionRegistry().registerAction(pasteObjectNetAction);
		registerHandleForEditorAction(ActionFactory.PASTE.getId());
		
		selectionAction = new FireDialogAction(getEditor());
		getActionRegistry().registerAction(selectionAction);
		registerHandleForEditorAction(FireDialogAction.ID);
		
		selectionAction = new ApplyRuleDialogAction(getEditor());
		getActionRegistry().registerAction(selectionAction);
		registerHandleForEditorAction(ApplyRuleDialogAction.ID);


	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#setupKeyHandler(org.eclipse.gef.KeyHandler)
	 */
	@Override
	protected void setupKeyHandler(final KeyHandler kh) {
		// Arrow keys run the correspondent MoveNodeActions
		kh.put(KeyStroke.getPressed(SWT.ARROW_LEFT, 0), (getActionRegistry().getAction(MoveNodeAction.LEFT)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_RIGHT, 0), (getActionRegistry().getAction(MoveNodeAction.RIGHT)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_UP, 0), (getActionRegistry().getAction(MoveNodeAction.UP)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_DOWN, 0), (getActionRegistry().getAction(MoveNodeAction.DOWN)));

		// SHIFT + Arrow keys run the correspondent precise MoveNodeActions
		kh.put(KeyStroke.getPressed(SWT.ARROW_LEFT, SWT.SHIFT), (getActionRegistry().getAction(MoveNodeAction.PREC_LEFT)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_RIGHT, SWT.SHIFT), (getActionRegistry().getAction(MoveNodeAction.PREC_RIGHT)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_UP, SWT.SHIFT), (getActionRegistry().getAction(MoveNodeAction.PREC_UP)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_DOWN, SWT.SHIFT), (getActionRegistry().getAction(MoveNodeAction.PREC_DOWN)));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#createEditPartFactory()
	 */
	@Override
	protected EditPartFactory createEditPartFactory() {
		return new RONEditPartFactory();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#getContextMenuProvider()
	 */
	@Override
	protected ContextMenuProviderWithActionRegistry createContextMenuProvider(final EditPartViewer viewer) {
		return new RONEditorContextMenuProvider(viewer, getActionRegistry());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#createPaletteRoot()
	 */
	@Override
	protected PaletteRoot createPaletteRoot() {
		// ObjectNetEditorPalette
		return new RONPaletteRoot();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#getThumbSashWeights()
	 */
	@Override
	protected int[] getThumbSashWeights() {
		return new int[] { 3, 1 };
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.emf.common.notify.Adapter#notifyChanged(org.eclipse.emf.common.notify.Notification)
	 */
	public void notifyChanged(final Notification notification) {
	
		// the page doesn't care for model changes (yet)
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.ruleview.AbstractMultiViewerPage#getNumberOfViewers()
	 */
	@Override
	public int getNumberOfViewers() {
		return 1;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.ruleview.AbstractMultiViewerPage#getViewerContents()
	 */
	@Override
	public EObject[] getViewerContents() {
		return new EObject[] { getModel() };
	}

	public void propertyChanged(Object source, int propId) {
		System.out.println("RONPage.propertyChanged(" + source + ")");

		// TODO Auto-generated method stub

	}

}
