package roneditor.ui.part;

import java.io.FileNotFoundException;
import java.io.IOException;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.gef.ContextMenuProvider;
import org.eclipse.gef.EditPartFactory;
import org.eclipse.gef.ui.actions.DeleteAction;
import org.eclipse.gef.ui.parts.TreeViewer;
import org.eclipse.jface.action.IAction;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.plugin.AbstractUIPlugin;

import roneditor.IRonEditorConstants;
import roneditor.RONActivator;
import roneditor.gef.actions.objectnet.FireTransitionAction;
import roneditor.gef.actions.objectnet.ObjectNetEditPartGraphLayoutAction;
import roneditor.gef.actions.ron.ApplyRulesAction;
import roneditor.gef.actions.ron.CreateHLPlaceAction;
import roneditor.gef.actions.ron.CreateTokenAction;
import roneditor.gef.editparts.tree.RONTreeEditPartFactory;
import roneditor.model.IDUtil;
import roneditor.model.RONModelManager;
import roneditor.ui.contextmenu.RONEditorContextMenuProvider;
import roneditor.ui.views.AbstractPageBookView;
import ronmodel.RON;

/**
 * This class extends {@link AbstractTreeEditor} to an editor for Reconfigurable
 * Object Nets (RONs).
 * 
 * @author Tony Modica
 */
public class RONTreeEditor extends AbstractTreeEditor {

	/**
	 * The manager for model persistency operations.
	 */
	private final RONModelManager modelManager = new RONModelManager();

	/**
	 * The root RON element of the model.
	 */
	private RON modelRoot = null;

	public RONTreeEditor() {
		// setTitleImage(RONActivator.getDefault().getImageRegistry().get(IMAGEKEY);
		super();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.editor.AbstractTreeEditor#dispose()
	 */
	@Override
	public void dispose() {
		final IWorkbenchPage activePage = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
		IDUtil.page = activePage;
		// activepage kann hier schon null sein, wenn Eclipse beendet wird
		if (activePage != null) {

			// Schliesse alle Views dieses Editors
			// Sonst gibt es Fehler, wenn noch ein anderer Editor offen ist.
			final IViewReference[] viewRefs = activePage.getViewReferences();
			for (final IViewReference element : viewRefs) {
				// close all Views that match the following ID
				if (element.getId().equals(IRonEditorConstants.objectNetViewID)) {
					activePage.hideView(element.getView(true));
				}
				// TODO: prüfen, ob das notwendig ist
				if (element.getId().equals(IRonEditorConstants.ruleViewID)) {
					activePage.hideView(element.getView(true));
				}
				if (element.getId().equals(IRonEditorConstants.ronViewID)) {
					activePage.hideView(element.getView(true));
				}
			}
		}
		super.dispose();
	}

	/**
	 * @return The RON that is the model's root object.
	 */
	/*
	 * 
	 * (non-Javadoc)
	 * 
	 * @see roneditor.editor.AbstractTreeEditor#getModelRoot()
	 */
	@Override
	public RON getModelRoot() {
		return modelRoot;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.editor.AbstractTreeEditor#getTreeEditPartFactory()
	 */
	@Override
	protected EditPartFactory createTreeEditPartFactory() {
		return new RONTreeEditPartFactory();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.editor.AbstractTreeEditor#getContextMenuProvider(org.eclipse.gef.ui.parts.TreeViewer)
	 */
	@Override
	protected ContextMenuProvider createContextMenuProvider(final TreeViewer viewer) {
		return new RONEditorContextMenuProvider(viewer, getActionRegistry());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.editor.AbstractTreeEditor#createActions()
	 */
	@Override
	protected void createActions() {
		// HINT Hier werden Instanzen der Actions erzeugt und registriert,
		// damit u.a. der ContextMenuProvider sie in das Kontextmen� einf�gen
		// kann.

		IAction action = new CreateHLPlaceAction(this);
		getActionRegistry().registerAction(action);
		getSelectionActions().add(action);

		action = new CreateTokenAction(this);
		getActionRegistry().registerAction(action);
		getSelectionActions().add(action);

		action = new ApplyRulesAction(this);
		getActionRegistry().registerAction(action);
		getSelectionActions().add(action);

		action = new DeleteAction((IWorkbenchPart) this);
		getActionRegistry().registerAction(action);
		getSelectionActions().add(action);

		action = new ObjectNetEditPartGraphLayoutAction(this);
		getActionRegistry().registerAction(action);
		getSelectionActions().add(action);

		action = new FireTransitionAction(this);
		getActionRegistry().registerAction(action);
		getSelectionActions().add(action);

		super.createActions();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.editor.AbstractTreeEditor#save(org.eclipse.core.resources.IFile,
	 *      org.eclipse.core.runtime.IProgressMonitor)
	 */
	@Override
	protected void save(final IFile file, final IProgressMonitor monitor) throws CoreException {

		monitor.beginTask("Saving " + file, 2);
		if (null == modelManager) {
			logError("No model modelManager found for saving the file.", null);
		}
		// save model to file
		try {
			modelManager.save(file.getFullPath());
			monitor.worked(1);
			file.refreshLocal(IResource.DEPTH_ZERO, new SubProgressMonitor(monitor, 1));
			monitor.done();
		} catch (final FileNotFoundException e) {
			logError("Error writing file.", e);
		} catch (final IOException e) {
			logError("Error writing file.", e);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.EditorPart#setInput(org.eclipse.ui.IEditorInput)
	 */
	@Override
	protected void setInput(final IEditorInput input) {
		super.setInput(input);
		final IFile file = ((IFileEditorInput) input).getFile();
		setPartName(file.getName());
		try {
			modelManager.load(file.getFullPath());
		} catch (final Exception e) {
			// Ist aus der Datei kein Modell lesbar, z.B. weil sie gerade durch
			// einen Wizard erzeugt wurde und noch leer ist, erzeugen wir direkt
			// ein default RON. Fehlermeldungen werden schon im
			// RonModelManager ausgegeben.
			modelManager.createDefaultRon(file.getFullPath());
		}
		modelRoot = modelManager.getRON();
		if (null == modelRoot) {
			logError("The loaded or created model is corrupt!", null);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.editor.AbstractTreeEditor#getActivator()
	 */
	@Override
	public AbstractUIPlugin getActivator() {
		return RONActivator.getDefault();
	}

	/**
	 * Extended to provide a {@link Resource} i.e. for
	 * {@link AbstractPageBookView}s.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.editor.AbstractTreeEditor#getAdapter(java.lang.Class)
	 */
	@Override
	public Object getAdapter(final Class type) {
		if (type == Resource.class) {
			return getModelRoot().eResource();
		}
		return super.getAdapter(type);
	}

}
