package roneditor.ui.utils;

import java.net.URL;
import java.util.HashMap;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;

import ronmodel.HLPlaceType;
import ronmodel.HLTransitionKind;
import ronmodel.RonmodelPackage;

/**
 * Singleton helper class to get an Icon for a requested item.
 * This class holds ALL icons used with the project. 
 * 
 * Please do not create Images for the icons on other places.  
 */
public class RONIconUtil {
	private static RONIconUtil iconUtil = null;
	
	private static final int ID_START = 1024; 
	
	public static final int MAGIC_ARC = ID_START;
	public static final int FIRE_TRANSITION = ID_START + 1;
	public static final int GRAPH_LAYOUT = ID_START + 2;
	public static final int HL_TRANSITION = ID_START + 3;
	public static final int TOKEN = ID_START + 4;

	private HashMap<ImageDescriptor, Image> icons;
	private HashMap<String, ImageDescriptor> descriptors;

	/**
	 * Return the instance of the {@link RONIconUtil}.
	 * @return
	 */
	public static RONIconUtil getInstance() {
		if (iconUtil == null) {
			iconUtil = new RONIconUtil();
		}
		return iconUtil;
	}

	public RONIconUtil() {
		icons = new HashMap<ImageDescriptor, Image>();
		descriptors = new HashMap<String, ImageDescriptor>();
	}
	
	/**
	 * Get a {@link ImageDescriptor} for the given name and with the given size
	 * @param name
	 * @param size
	 * @return
	 */
	public ImageDescriptor getDescriptor(String name, int size) {
		String fileName = name + size + ".png";
		ImageDescriptor result = descriptors.get(fileName);
		if (result != null)
			return result;
		
		URL url = getClass().getResource("/icons/" + fileName);
		result = ImageDescriptor.createFromURL(url);
		descriptors.put(fileName, result);
		return result;
		
	}
	
	/**
	 * Get a {@link Image} for the given {@link ImageDescriptor}.
	 * The entries are cached in a {@link HashMap}. 
	 * 
	 * @param name
	 * @param size
	 * @return
	 */
	public Image getIcon(ImageDescriptor descriptor) {
		Image icon = icons.get(descriptor);
		if ( icon != null ) return icon;
		icon = descriptor.createImage();
		icons.put(descriptor, icon);
		return icon;
	}

	/**
	 * Get a {@link Image} for the given name and with the given size
	 * @param name
	 * @param size
	 * @return
	 */
	public Image getIcon(String name, int size) {
		 return getIcon(getDescriptor(name, size));
	}
	
	/**
	 * Get a {@link Image} for the given type and with the given size
	 * @param name
	 * @param size
	 * @return
	 */
	public Image getIcon(int type, int size) {
		return getIcon(getDescriptor(type, size));
	}

	/**
	 * Get a {@link Image} for the given type and a special attribute with the given size
	 * @param name
	 * @param size
	 * @return
	 */
	public Image getIcon(int type, int attribute, int size) {
		return getIcon(getDescriptor(type, attribute, size));
	}
		
	/**
	 * Get a {@link ImageDescriptor} for the given type and with the given size
	 * @param name
	 * @param size
	 * @return
	 */
	public ImageDescriptor getDescriptor(int type, int size) {
		return getDescriptor(type, 0, size);
	}
	
	
	/**
	 * Get a {@link ImageDescriptor} for the given type and a special attribute with the given size
	 * @param name
	 * @param size
	 * @return
	 */
	public ImageDescriptor getDescriptor(int type, int attribute, int size) {
		switch (type) {
		case RonmodelPackage.OBJECT_NET:
			return getDescriptor("objectnet", size);
		case RonmodelPackage.RULE:
			return getDescriptor("ruleedit", size);
		case RonmodelPackage.HL_PLACE:
			switch (attribute) {
			case HLPlaceType.NET:
				return getDescriptor("hlplaceobjectnet", size);
			case HLPlaceType.RULE:
				return getDescriptor("hlplacerule", size);
			}
		case RonmodelPackage.TOKEN:
			switch (attribute) {
			case HLPlaceType.NET:
				return getDescriptor("objectnet", size);
			case HLPlaceType.RULE:
				return getDescriptor("ruleedit", size);
			}
		case RonmodelPackage.ON_ARC:
		case RonmodelPackage.HL_ARC:
			return getDescriptor("onarc", size);
		case RonmodelPackage.ON_PLACE:
			return getDescriptor("onplace", size);
		case RonmodelPackage.ON_TRANSITION:
			return getDescriptor("ontransition", size);
		case RonmodelPackage.MAPPING:
			return getDescriptor("mapping", size);
		case RonmodelPackage.RON:
			return getDescriptor("ron", size);
		case RonmodelPackage.HL_TRANSITION:
			switch (attribute) {
			case HLTransitionKind.APPLYRULE:
				return getDescriptor("transition-apply", size);
			case HLTransitionKind.FIRE:
				return getDescriptor("transition-fire", size);
			case HLTransitionKind.SPLIT:
				return getDescriptor("transition-split", size);
			case HLTransitionKind.STANDARD:
				return getDescriptor("transition-standard", size);
			}
		case HL_TRANSITION:
			switch (attribute) {
			case HLTransitionKind.APPLYRULE:
				return getDescriptor("apply", size);
			case HLTransitionKind.FIRE:
				return getDescriptor("fire", size);
			case HLTransitionKind.SPLIT:
				return getDescriptor("split", size);
			case HLTransitionKind.STANDARD:
				return getDescriptor("standard", size);
			}
		case FIRE_TRANSITION:
			return getDescriptor("fireicon", size);
		case MAGIC_ARC:
			return getDescriptor("magicarc", size);		
		case GRAPH_LAYOUT:
			return getDescriptor("graphlayout", size);
		case TOKEN:
			return getDescriptor("token", size);
		}
		return getDescriptor("unknown", 16);
	}

	
	
}
